import java.util.*;

public class Problem {
    final KeyRing handKeys;
    final List<Chest> chests;
    final Collection<Key> chestKeys;
    int[] totalKeyCount = new int[400];
    int[] totalChestCount = new int[400];
    int maxColor = 0;

    final int[] closedChestsByColor = new int[400];

    ReachableColors reachableColors = null;
    // Edge from -1 to c means keys of color c in hand.
    // edge from c to d means chest of color c contains keys of color d.
    private final Map<Integer, Map<Integer, Integer>> edgeCounts =
        new HashMap<Integer, Map<Integer, Integer>>();
    private void addEdges(Integer from, Integer to, int howMany) {
        Map<Integer, Integer> map = edgeCounts.get(from);
        if (map == null) map = new TreeMap<Integer, Integer>();
        Integer count = map.get(to);
        if (count == null) count = 0;
        map.put(to, count + howMany);
        edgeCounts.put(from, map);
    }
    // true if it zeroes out this from/to.
    private boolean removeEdges(Integer from, Integer to, int howMany) {
        if (howMany == 0) {
            return false;
        }
        Map<Integer, Integer> map = edgeCounts.get(from);
        Integer count = map.get(to);
        if (count == null || count < howMany) {
            throw new RuntimeException("remove too many edges: " + from + " " + to + " " + howMany + " > " + count);
        }
        count-= howMany;
        if (count == 0) {
            map.remove(to);
            return true;
        } else {
            map.put(to, count);
            return false;
        }
    }
    private static void debug(Object o) {
        String s = o.toString();
        try {
            System.err.println(s);
        } catch (Exception e) {
            // do nothing
        }
    }

    public Problem(String[] keysLine,
                   List<String[]> chestLines) throws Exception {
        handKeys = new KeyRing(keysLine.length);
        chestKeys = new ArrayList<Key>(400);

        for (String s : keysLine) {
            Key k = new Key(Integer.parseInt(s), -1);
            handKeys.add(k);
            addEdges(-1, k.color, 1);
            totalKeyCount[k.color]++;
            maxColor = Math.max(k.color, maxColor);
        }
        chests = new ArrayList<Chest>(chestLines.size());
        int chestName = 1;
        for (String[] chestLine : chestLines) {
            Integer chestColor = Integer.parseInt(chestLine[0]);
            Integer numKeys = Integer.parseInt(chestLine[1]);
            List<Key> loot = new ArrayList<Key>(numKeys);
            for (int i = 0; i < numKeys; i++) {
                Key k = new Key(Integer.parseInt(chestLine[2 + i]),
                                chestName - 1);
                totalKeyCount[k.color]++;
                maxColor = Math.max(k.color, maxColor);
                loot.add(k);
                addEdges(chestColor, k.color, 1);
            }
            totalChestCount[chestColor]++;
            chests.add(new Chest(chestName, chestColor, loot));
            closedChestsByColor[chestColor]++;
            chestName++;
        }
        reachableColors = new ReachableColors();
    }
    public String solve() {
        if (isTriviallyStuck() || isStuck(null)) {
            return "IMPOSSIBLE";
        } else {
            StringBuilder sb = new StringBuilder();
            while(!isWin()) {
                // Extend the solution chain, left-first DFS
                // dummy var
                Move lastMoveTried = new Move();
                do {
                    lastMoveTried.undo();
                    int chestToAdd = getFirstChestAfter(lastMoveTried.chestOpened);
                    if (chestToAdd < 0) {
                        throw new RuntimeException("Can't open any box after "
                                                   + sb);
                    }
                    lastMoveTried = openChest(chestToAdd);
                } while (isStuck(lastMoveTried.keyUsed));
                if (sb.length() != 0) sb.append(" ");
                sb.append(chests.get(lastMoveTried.chestOpened).name);
            }
            return sb.toString();
        }
    }
    
    private class Key {
        public final Integer color;
        public final Integer location;
        public Key(Integer c, Integer l) {
            this.color = c;
            this.location = l;
        }
        @Override
        public String toString() {
            return "Key of " + color + " in " + location;
        }
    }
    private class KeyRing {
        //K private Collection<Key> keys;
        final Map<Integer,ArrayList<Key>> colorCounts = new TreeMap<Integer, ArrayList<Key>>();
        public KeyRing(int expectedSize) {
            //K keys = new ArrayList<Key>(expectedSize);
        }
        public KeyRing() {
            this(400);
        }
        public void add(Key k) {
            //K keys.add(k);
            ArrayList<Key> oldCount = colorCounts.get(k.color);
            if (oldCount == null) oldCount = new ArrayList<Key>();
            oldCount.add(k);
            colorCounts.put(k.color, oldCount);            
        }
        public void addAll(Collection<Key> ks) {
            for (Key k : ks) {
                add(k);
            }
        }
        public boolean containsKeyIn(Collection<Integer> targets) {
            for (Integer color : targets) {
                if (containsKeyIn(color)) return true;
            }
            return false;
        }
        public boolean containsKeyIn(Integer color) {
            ArrayList<Key> count = colorCounts.get(color);
            if (count != null && !count.isEmpty()) return true;
            return false;
        }
        public Key removeKeyOf(Integer color) {
            ArrayList<Key> count = colorCounts.get(color);
            return count.remove(0);
        }
    }
    private class Chest {
        public final Integer name;
        public final Integer color;
        public Map<Integer, Integer> loot = new TreeMap<Integer, Integer>();

        public boolean open = false;
        
        public Chest(Integer name, Integer color, Collection<Key> l) {
            this.name = name;
            this.color = color;
            for (Key k : l) {
                Integer oldCount = loot.get(k.color);
                if (oldCount == null) oldCount = 0;
                loot.put(k.color, oldCount + 1);
            }
        }
        /*        
        public boolean containsKeyIn(Collection<Integer> colors) {
            for (Integer c : lootColors) {
                if (colors.contains(c)) {
                    return true;
                }
            }
            return false;
        }
        */
        @Override
        public String toString() {
            return "Chest" + name;
        }
    }
    private class ReachableColors {
        private final Collection<Integer> reachable;
        private final boolean complete;
        public ReachableColors() {
            reachable = new HashSet<Integer>(maxColor);
            reachable.add(-1); // to get into hand
            HashSet<Integer> newly;
            do {
                newly = new HashSet<Integer>(maxColor);
                for (Integer c : reachable) {
                    Map<Integer, Integer> toMap = edgeCounts.get(c);
                    if (toMap != null) {
                        for (Map.Entry<Integer, Integer> e :
                                 toMap.entrySet()) {
                            if (e.getValue() > 0) {
                                Integer d = e.getKey();
                                if (!reachable.contains(d)) {
                                    newly.add(d);
                                }
                            }
                        }
                    }
                }
                reachable.addAll(newly);
            } while (newly.size() > 0);
            complete = isComplete();
        }
        private boolean isComplete() {
            for (int i = 0; i <= maxColor; i++) {
                if (closedChestsByColor[i] > 0 && !reachable.contains(i)) {
                    return false;
                }
            }
            return true;
        }
        public boolean reachesAll() {
            return complete;
        }
    }





    private Move openChest(int which) {
        return new Move(which);
    }

    private class Move {
        public final Integer chestOpened;
        public final Integer keyUsed;
        private final ReachableColors oldRC;
        // A no-op move
        public Move() {
            this(-1);
        }
        // has side effects, all reversed by undo()
        public Move(int whichOpened) {
            chestOpened = whichOpened;
            if (whichOpened == -1) {
                keyUsed = null;
                oldRC = null;
                return;
            }
            
            Chest chest = chests.get(whichOpened);
            if (chest.open) {
                throw new IllegalArgumentException("Chest "+ whichOpened + " already open");
            }
            keyUsed = chest.color;

            chest.open = true;
            closedChestsByColor[chest.color]--;
            boolean graphCut = false;
            for (Map.Entry<Integer, Integer> keyCount : chest.loot.entrySet()) {
                addEdges(-1, keyCount.getKey(), keyCount.getValue());
                // don't short circuit!

            }
            graphCut = removeEdges(-1, chest.color, 1) || graphCut;
            if (graphCut) {
                oldRC = reachableColors;
                reachableColors = new ReachableColors();
            } else {
                oldRC = null;
            }
            // TODO: more side effects
        }
        void undo() {
            if (chestOpened == -1) {
                return;
            }
            Chest chest = chests.get(chestOpened);
            
            chest.open = false;
            closedChestsByColor[chest.color]++;
            for (Map.Entry<Integer, Integer> keyCount : chest.loot.entrySet()) {
                removeEdges(-1, keyCount.getKey(), keyCount.getValue());
            }
            addEdges(-1, chest.color, 1);

            if (oldRC != null) {
                reachableColors = oldRC;
            }
        }
    }

    private boolean isTriviallyStuck() {
        for (int c = 0; c <= maxColor; c++) {
            if (totalKeyCount[c] < totalChestCount[c]) {
                return true;
            }
        }
        return false;
    }
    
    private boolean isStuck(Integer hintKeyColorUsed) {
        // TODO: use keyHint to fail faster
        return !reachableColors.reachesAll();
    }
    private boolean isWin() {
        for (Chest c : chests) {
            if (!c.open) {
                return false;
            }
        }
        return true;
    }
    private boolean haveHandKeyFor(Integer color) {
        Integer n = edgeCounts.get(-1).get(color);
        if (n == null) return false;
        return true;
    }

    private int getFirstChestAfter(int afterWhat) {
        while (++afterWhat < chests.size()) {
            Chest c = chests.get(afterWhat);
            if (!c.open && haveHandKeyFor(c.color)) return afterWhat;
        }
        return -1;
    }
}