package aditsu.gcj.portal;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public abstract class AStar<T> {
	protected final Map<T, Integer> posMap = new HashMap<T, Integer>();
	protected final Heap<AStarInfo<T>> heap = new Heap<AStarInfo<T>>();
	protected final Set<T> closed = new HashSet<T>();

	private final HeapListener<AStarInfo<T>> listener = new HeapListener<AStarInfo<T>>() {
		public void added(final AStarInfo<T> x, final int pos) {
			posMap.put(x.node, pos);
		}

		public void moved(final AStarInfo<T> x, final int oldPos, final int newPos) {
			posMap.put(x.node, newPos);
		}

		public void removed(final AStarInfo<T> x, final int pos) {
			posMap.remove(x.node);
			closed.add(x.node);
		}
	};

	public AStar() {
		heap.setListener(listener);
	}

	protected List<T> expandNode(final T node) {
		throw new UnsupportedOperationException();
	}

	protected List<AStarInfo<T>> expand(final AStarInfo<T> inf) {
		final List<T> nl = expandNode(inf.node);
		final List<AStarInfo<T>> il = new ArrayList<AStarInfo<T>>(nl.size());
		for (T t : nl) {
			il.add(makeInfo(inf, t));
		}
		return il;
	}

	protected abstract boolean solved(final T node);

	protected int heuristic(final T node) {
		throw new UnsupportedOperationException();
	}

	protected AStarInfo<T> makeInfo(final AStarInfo<T> parent, final T node) {
		final int dist = parent == null ? 0 : parent.dist + 1;
		return new AStarInfo<T>(node, dist, heuristic(node), parent);
	}

	public AStarInfo<T> run(final T node) {
		return run(makeInfo(null, node));
	}

	public AStarInfo<T> run(final AStarInfo<T> inf0) {
		heap.insert(inf0);
		int c = 0;
		while (true) {
			if (heap.isEmpty()) {
				return null;
			}
			final AStarInfo<T> inf = heap.remove(0);
			if (solved(inf.node)) {
				return inf;
			}
			if (++c == 1000) {
				System.out.println("Open: " + posMap.size() + "/" + heap.getSize() + ", closed: " + closed.size());
				System.out.println(inf);
				c = 0;
			}
			final List<AStarInfo<T>> l = expand(inf);
			for (AStarInfo<T> inf1 : l) {
				if (closed.contains(inf1.node)) {
					continue;
				}
				final Integer pos = posMap.get(inf1.node);
				if (pos != null) {
					final AStarInfo<T> inf2 = heap.get(pos);
					if (inf2.total() <= inf1.total()) {
						continue;
					}
					heap.remove(pos);
				}
				heap.insert(inf1);
			}
		}
	}

	public List<T> getPath(final AStarInfo<T> inf) {
		final List<T> l = new ArrayList<T>();
		AStarInfo<T> x = inf;
		while (x != null) {
			l.add(x.node);
			x = x.parent;
		}
		Collections.reverse(l);
		return l;
	}
}
