/**
 * The source code in this file is compilable and runnable with the Java SE
 * JDK 1.6.0_07 downloadable from http://java.sun.com/javase/downloads/index.jsp
 */
package gcj.lorenzfischer;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.PriorityQueue;
import java.util.Queue;

/**
 * Solution to the "Train Timetable" puzzle for Google Codjam 2008.
 * 
 * In order to keep things simple I use java.util.Date instances for the
 * arrival/departure times. The dates are all January 1, 1970.
 * 
 * @author lorenz.fischer@gmail.com
 */
public class TrainTimetable {

	public final int trainsA;
	public final int trainsB;

	public TrainTimetable(int turnAroundTime, List<TravelTime> trainsFromA, List<TravelTime> trainsFromB) {
		trainsA = trainsNeeded(turnAroundTime, getDepartures(trainsFromA), getArrivals(trainsFromB));
		trainsB = trainsNeeded(turnAroundTime, getDepartures(trainsFromB), getArrivals(trainsFromA));
	}

	/** @return a queue containing the sorted arrival times from entryList */
	private Queue<Date> getArrivals(List<TravelTime> entryList) {
		final PriorityQueue<Date> result = new PriorityQueue<Date>();
		for (TravelTime tt : entryList) {
			result.offer(tt.arrival);
		}
		return result;
	}

	/** @return a queue containing the sorted departure times from entryList */
	private Queue<Date> getDepartures(List<TravelTime> entryList) {
		final PriorityQueue<Date> result = new PriorityQueue<Date>();
		for (TravelTime tt : entryList) {
			result.offer(tt.departure);
		}
		return result;
	}

	private int trainsNeeded(int turnaroundTime, Queue<Date> departures, Queue<Date> arrivals) {
		int countNewTrains = 0;

		Date nextArrivalReady = nextArrivalReady(turnaroundTime, arrivals);
		while (departures.size() > 0) {
			Date d = departures.poll();
			if ((nextArrivalReady != null) && (d.compareTo(nextArrivalReady) >= 0)) {
				// we can use a train that has arrived
				nextArrivalReady = nextArrivalReady(turnaroundTime, arrivals);
			} else {
				// we need a new train
				countNewTrains++;
			}
		}

		return countNewTrains;
	}

	/**
	 * @return the date for the next train that will have arrived and is ready,
	 *         null if no train is ready for departure
	 */
	private Date nextArrivalReady(int turnaroundTime, Queue<Date> arrivals) {
		Date result = arrivals.poll();

		if (result != null) {
			// add turnaround time
			result = new Date(result.getTime() + (turnaroundTime * 60 * 1000));
		}

		return result;
	}

	/**
	 * @param args
	 *            arguments
	 * @throws IOException if input/output files can't be read/written from/to
	 */
	public static void main(String[] args) throws IOException {
		File inputFile = new File("B-large.in");
		BufferedReader br = new BufferedReader(new FileReader(inputFile));
		File outputFile = new File("B-large.out");
		BufferedWriter wr = new BufferedWriter(new FileWriter(outputFile));

		int numberOfCases = Integer.valueOf(br.readLine()).intValue();

		for (int i = 0; i < numberOfCases; i++) {
			long time = System.currentTimeMillis();
			int turnaroundTime = Integer.valueOf(br.readLine()).intValue();
			String[] numberOfTrains = br.readLine().split(" ");
			int na = Integer.valueOf(numberOfTrains[0]).intValue();
			int nb = Integer.valueOf(numberOfTrains[1]).intValue();
			List<TravelTime> trainsFromA = new ArrayList<TravelTime>();
			List<TravelTime> trainsFromB = new ArrayList<TravelTime>();
			TrainTimetable timeTable;

			System.out.print("starting case #" + (i + 1) + " ...");

			for (int j = 0; j < na; j++) {
				trainsFromA.add(new TravelTime(br.readLine()));
			}

			for (int j = 0; j < nb; j++) {
				trainsFromB.add(new TravelTime(br.readLine()));
			}

			timeTable = new TrainTimetable(turnaroundTime, trainsFromA, trainsFromB);

			wr.write("Case #" + (i + 1) + ": " + timeTable.trainsA + " " + timeTable.trainsB + "\n");

			System.out.print(" done! took " + (System.currentTimeMillis() - time) + " millis.\n");

		}

		br.close();
		wr.close();
	}

	/**
	 * This class is just a container for the departure and arrival time of a
	 * train.
	 */
	public static class TravelTime {

		private final DateFormat df = new SimpleDateFormat("HH:mm");
		private final Date departure;
		private final Date arrival;

		/**
		 * @param input
		 *            a String of the form "HH:mm HH:mm".
		 */
		public TravelTime(String input) {
			String[] inputVals = input.split(" ");
			try {
				this.departure = df.parse(inputVals[0]);
				this.arrival = df.parse(inputVals[1]);
			} catch (ParseException e) {
				throw new RuntimeException(e); // we don't want to go on
			}
		}
	}
}
