import java.io.*;
import java.util.*;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

/**
 * Usage: java MainClassName < fileIn.txt > fileOut.txt
 */
public class GCJ2017QA {

    private static class BitSet implements Serializable {
        final int size;
        int bitsSet;
        boolean data[];

        public BitSet(int size) {
            this.size = size;
            data = new boolean[size];
        }

        boolean isSet(int idx) {
            return data[idx];
        }

        void set(int idx) {
            if (!data[idx]) bitsSet++;
            data[idx] = true;
        }

        void unset(int idx) {
            if (data[idx]) bitsSet--;
            data[idx] = false;
        }

        public int flip(int idx) {
            if (isSet(idx)) {
                unset(idx);
                return 0;
            } else {
                set(idx);
                return 1;
            }
        }

        public void flipKBits(int beginIdx, int k) {
            for (int i = 0; i < k; i++) {
                flip(beginIdx + i);
            }
        }

        public boolean isAllSet() {
            return size == bitsSet;
        }

        public static BitSet parseString(String str) {
            BitSet bitSet = new BitSet(str.length());
            for (int i = 0; i < str.length(); i++) {
                if (str.charAt(i) == '+') {
                    bitSet.set(i);
                }
            }
            return bitSet;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            BitSet bitSet = (BitSet) o;

            if (size != bitSet.size) return false;
            if (bitsSet != bitSet.bitsSet) return false;
            return Arrays.equals(data, bitSet.data);
        }

        @Override
        public int hashCode() {
            int result = size;
            result = 31 * result + bitsSet;
            result = 31 * result + Arrays.hashCode(data);
            return result;
        }

        public BitSet copy() {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            try {
                ObjectOutputStream oos = new ObjectOutputStream(bos);
                oos.writeObject(this);
                oos.flush();
                oos.close();
                bos.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            byte[] byteData = bos.toByteArray();

            ByteArrayInputStream bais = new ByteArrayInputStream(byteData);
            try {
                return (BitSet) new ObjectInputStream(bais).readObject();
            } catch (IOException | ClassNotFoundException e) {
                e.printStackTrace();
            }

            return null;
        }

        @Override
        public String toString() {
            return "BitSet{" +
                    "size=" + size +
                    ", bitsSet=" + bitsSet +
                    ", data=" + Arrays.toString(data) +
                    '}';
        }
    }

    private static class Position {
        private final long movesMade;
        private final BitSet bitSet;

        private Position(long movesMade, BitSet bitSet) {
            this.movesMade = movesMade;
            this.bitSet = bitSet;
        }
    }


    private static class InputData {
        int K;
        BitSet pancakes;

        InputData(Scanner in) {
            pancakes = BitSet.parseString(in.next());
            K = in.nextInt();
        }

        /**
         * Main method to solve the problem
         */
        void solve(PrintWriter out) {
            Position start = new Position(0, pancakes);
            long moves = getMinMoves(start);
            if (moves == -1) {
                out.println("IMPOSSIBLE");
            } else {
                out.println(moves);
            }
        }

        private long getMinMoves(Position start) {
            Set<BitSet> visited = new HashSet<>();
            Queue<Position> availableMoves = new PriorityQueue<Position>(1, new Comparator<Position>() {
                @Override
                public int compare(Position o1, Position o2) {
                    long d1 = o1.movesMade + (o1.bitSet.size - o1.bitSet.bitsSet);
                    long d2 = o2.movesMade + (o2.bitSet.size - o2.bitSet.bitsSet);
                    return Long.compare(d1, d2);
                }
            });
            visited.add(start.bitSet);
            availableMoves.offer(start);
            while (!availableMoves.isEmpty()) {
                Position pos = availableMoves.poll();
                if (pos.bitSet.isAllSet()) {
                    return pos.movesMade;
                }
                addNewMoves(availableMoves, pos, visited);
            }
            return -1;
        }

        private void addNewMoves(Queue<Position> moves, Position pos, Set<BitSet> visited) {
            for (int i = 0; i < pos.bitSet.size - K + 1; i++) {
                BitSet copy = pos.bitSet.copy();
                int newBitsSet = 0;
                for (int j = 0; j < K; j++) {
                    newBitsSet += copy.flip(i + j);
                }
                // todo: if new position just reverted everything - then it doesn't make sense?
                if (visited.add(copy)) {
                    moves.offer(new Position(pos.movesMade + 1, copy));
                }
            }
        }
    }

    static class Solver implements Callable<String> {

        InputData data;

        Solver(InputData data) {
            this.data = data;
        }

        @Override
        public String call() {
            StringWriter out = new StringWriter();
            data.solve(new PrintWriter(out));
            return out.toString();
        }

    }

    public static void main(String[] args) {
        try {
            // Scanner in = new Scanner(new File("C:\\Dropbox\\java\\projects\\google-codejam-2014\\inputs\\qualification\\Problem C. Minesweeper Master - small.txt"));
            Scanner in = new Scanner(System.in);

            ExecutorService service = Executors.newFixedThreadPool(NUM_THREADS);

            int testCount = in.nextInt();
            @SuppressWarnings("unchecked")
            Future<String>[] ts = new Future[testCount];
            for (int test = 0; test < testCount; test++) {
                ts[test] = service.submit(new Solver(new InputData(in)));
            }
            for (int test = 0; test < testCount; ++test) {
                while (!ts[test].isDone()) {
                    Thread.sleep(500);
                }
                System.err.println("Test " + (test + 1));
                out.print("Case #" + (test + 1) + ": ");
                out.print(ts[test].get());
            }
            service.shutdown();

            out.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private final static PrintStream out = System.out;
    private final static int NUM_THREADS = 10;
}
