import java.io.*;

/**
 * Template to read multiple tasks and solve them
 */
public class Solver {

    private class Task {
        public int length;
        public int repeat;
        public String s;
    }

    private class Solution {
        boolean canProve;

        public Solution(boolean canProve) {
            this.canProve = canProve;
        }

        @Override
        public String toString() {
            return canProve?"YES":"NO";
        }
    }

    final InputStream in;
    final PrintStream out;

    public Solver(InputStream in, PrintStream out) {
        this.in = in;
        this.out = out;
    }

    /**
     * Read input.
     * First line is number of tasks.
     * Parse tasks and solve them.
     * @throws IOException if reading fails
     */
    public void solveAll() throws IOException {
        prepareMulMatrix();

        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        int cases = Integer.parseInt(reader.readLine());
        for(int i=0;i<cases;i++) {
            Task task = parseTask(reader);
            out.println("Case #" + (i + 1) + ": " + solve(task).toString());
        }
    }
/*
    private void expr(String x, String y) {
        System.out.println(x + " * " + y + " = " + mc(x, y));
    }

    private String mc(String x, String y) {
        return decode(multiply(encode(x), encode(y)));
    }
*/

    private Task parseTask(BufferedReader reader) throws IOException {
        Task task = new Task();
        int []params = parseIntArray(reader.readLine());
        task.length = params[0];
        task.repeat = params[1];
        task.s = reader.readLine();
        return task;
    }

    int[] toFind = {encode('i'), encode('j'), encode('k')};

    private Solution solve(Task task) {
        int string[] = buildString(task.s, task.repeat);
        return new Solution(matchStrings(string, 0, toFind, 0));
    }

    private boolean matchStrings(int[] string, int from, int[] sample, int pos) {
        if (pos==sample.length) return from==string.length; // end sample, end string
        int nextStringPos = from;
        int initial = 1;
        do {
            nextStringPos = find(string, nextStringPos, initial, sample[pos]);
            if (nextStringPos==-1) return false;
            initial = sample[pos];
        } while(!matchStrings(string, nextStringPos, sample, pos + 1));
        return true;
    }

    // -1 means can't find, otherwise position of next char
    private int find(int[] string, int from, int initial, int value) {
        if (from>=string.length) return -1;
        int prod = initial;
        do {
            prod = multiply(prod, string[from]);
            from++;
        } while(prod!=value && from<string.length);
        return prod==value?from:-1;
    }

    private int[] buildString(String s, int repeat) {
        int len = s.length();
        int encoded[] = new int[len*repeat];
        for(int i=0;i<len;i++) {
            encoded[i] = encode(s.charAt(i));
        }
        for(int i=1;i<repeat;i++) {
            System.arraycopy(encoded, 0, encoded, i*len, len);
        }
        return encoded;
    }

    private int encode(char c) {
        if (c=='1') return 1;
        if (c=='i') return 2;
        if (c=='j') return 3;
        if (c=='k') return 4;
        throw new IllegalArgumentException("Bad character " + c);
    }

    private int encode(String s) {
        char c;
        int sign;
        if (s.length()==2) {
            sign = -1;
            c = s.charAt(1);
        } else {
            sign = 1;
            c = s.charAt(0);
        }
        if (c=='1') return 1*sign;
        if (c=='i') return 2*sign;
        if (c=='j') return 3*sign;
        if (c=='k') return 4*sign;
        if (c=='0') return 0;
        throw new IllegalArgumentException("Bad character " + c);
    }

    String[] positive = {"0","1","i","j","k"};
    String[] negative = {"0","-1","-i","-j","-k"};
    private String decode(int c) {
        return c>0?positive[c]:negative[-c];
    }


    String mul_matrix[] = {
    //           -k    -j    -i    -1     0     1     i     j     k
    /* -k */   "-1", "-i",  "j",  "k",  "0", "-k", "-j",  "i",  "1",
    /* -j */    "i", "-1", "-k",  "j",  "0", "-j",  "k",  "1", "-i",
    /* -i */   "-j",  "k", "-1",  "i",  "0", "-i",  "1", "-k",  "j",
    /* -1 */    "k",  "j",  "i",  "1",  "0", "-1", "-i", "-j", "-k",
    /*  0 */    "0",  "0",  "0",  "0",  "0",  "0",  "0",  "0",  "0",
    /*  1 */   "-k", "-j", "-i", "-1",  "0",  "1",  "i",  "j",  "k",
    /*  i */    "j", "-k",  "1", "-i",  "0",  "i", "-1",  "k", "-j",
    /*  j */   "-i",  "1",  "k", "-j",  "0",  "j", "-k", "-1",  "i",
    /*  k */    "1",  "i", "-j", "-k",  "0",  "k",  "j", "-i", "-1",
    };

    private void prepareMulMatrix() {
        mul = new int[mul_matrix.length];
        for(int i=0;i<mul_matrix.length;i++) {
            mul[i] = encode(mul_matrix[i]);
        }
    }

    int [] mul;
    private int multiply(int x, int y) {
        return mul[y+4+(x+4)*9];
    }

    // read descriptors (matrix sizes etc)
    public static int[] parseIntArray(String line) {
        String[] parts = line.trim().split(" +");
        int[] result = new int[parts.length];
        for(int i=0;i<parts.length;i++) {
            result[i] = Integer.parseInt(parts[i]);
        }
        return result;
    }

    public static void main(String[] args) throws IOException {
        InputStream in;
        PrintStream out;
        if (args.length>0) {
            in = new FileInputStream(args[0]);
        } else {
            in = System.in;
        }
        if (args.length>1) {
            out = new PrintStream(new FileOutputStream(args[1]));
        } else {
            out = System.out;
        }
        new Solver(in, out).solveAll();
    }
}
