import java.io.*;
import java.util.*;

/**
 * Template to read multiple tasks and solve them
 */
public class Solver {

    private class Task {
        int[] plates;

        public Task(int[] plates) {
            this.plates = plates;
        }
    }

    private class Solution {
        int moves;

        public Solution(int moves) {
            this.moves = moves;
        }

        @Override
        public String toString() {
            return Integer.toString(moves);
        }
    }

    final InputStream in;
    final PrintStream out;

    public Solver(InputStream in, PrintStream out) {
        this.in = in;
        this.out = out;
    }

    /**
     * Read input.
     * First line is number of tasks.
     * Parse tasks and solve them.
     * @throws IOException if reading fails
     */
    public void solveAll() throws IOException {
//        populateSplitTable();

        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        int cases = Integer.parseInt(reader.readLine());
        for(int i=0;i<cases;i++) {
            Task task = parseTask(reader);
            out.println("Case #" + (i + 1) + ": " + solve(task).toString());
        }
    }

//    private void populateSplitTable() {
//        for(int i=1;i<=1000;i++) {
//            maxMovesPerPlate(i);
//        }
//    }

    private Task parseTask(BufferedReader reader) throws IOException {
        reader.readLine(); // ignore count
        return new Task(parseIntArray(reader.readLine()));
    }


//    private final static double log2Base = Math.log(2);
//    static double log2(int x)
//    {
//        return Math.log(x) / log2Base;
//    }

    private Solution solve(Task task) {
        LinkedList<Integer> values = new LinkedList<Integer>();
        Arrays.sort(task.plates);
        for(int i=task.plates.length-1;i>=0;i--) values.add(task.plates[i]);
        int maxTime;
        int newMaxTime = values.getFirst();
        int splits = 0;
        do {
            maxTime = newMaxTime;
            // try split max
            int first = values.removeFirst();
            Moves moves = maxMovesPerPlate(first);
            if (moves.large<first) {
                int value1 = moves.large;
                int value2 = first - moves.large;
                addToList(values, value1, value2);
                // update counters
                splits++;
            }
            newMaxTime = first + splits;
        } while(newMaxTime<maxTime);
        return new Solution(maxTime);
    }

    private static class Moves implements Comparable<Moves> {
        public final int shared;
        public final int dedicated;
        public final int plate;
        public final int large;

        public Moves(int shared, int dedicated, int plate, int large) {
            this.shared = shared;
            this.dedicated = dedicated;
            this.plate = plate;
            this.large = large;
        }

        public int total() {
            return shared + dedicated;
        }

        @Override
        public String toString() {
            return shared + "+" + dedicated + " (" + large + ")";
        }

        public Moves addSplit(Moves other, int plate, int large) {
            return new Moves(Math.max(shared, other.shared), dedicated+other.dedicated+1, plate, large);
        }

        public int compareTo(Moves o) {
            if (o.total()>total()) return -1;
            if (o.total()<total()) return 1;
            if (o.dedicated>dedicated) return -1;
            if (o.dedicated<dedicated) return 1;
            return 0;
        }
    }

    private static final Moves IMPOSSIBLE = new Moves(99999999, 99999999, 0, 0);

    Map<Integer, Moves> minMoves = new HashMap<Integer, Moves>();

    private Moves maxMovesPerPlate(int plate) {
        if (plate==1) return new Moves(1, 0, 1, 1);
        if (minMoves.containsKey(plate)) return minMoves.get(plate);
        Moves min = new Moves(plate, 0, plate, plate);
        for(int i=2;i<1000;i++) {
            Moves split = minIfSplitBy(plate,i);
            if (min.compareTo(split)>0) min=split;
        }
        minMoves.put(plate,min);
        return min;
    }

    // min by total ?
    private Moves minIfSplitBy(int plate, int splits) {
        if (splits>plate) return IMPOSSIBLE;
        int piece1 = plate/splits;
        int piece2 = plate*(splits-1)/splits;
        if (piece1+piece2<plate) {
            // combination
            Moves type1 = maxMovesPerPlate(piece1+1).addSplit(maxMovesPerPlate(piece2), plate, piece2);
            Moves type2 = maxMovesPerPlate(piece1).addSplit(maxMovesPerPlate(piece2+1), plate, piece2+1);
            return type1.compareTo(type2)<0?type1:type2;
        } else {
            return maxMovesPerPlate(piece1).addSplit(maxMovesPerPlate(piece2), plate, piece2);
        }
    }

    // value1 > value2
    private void addToList(LinkedList<Integer> values, int newValue1, int newValue2) {
        ListIterator<Integer> iter = values.listIterator();
        while(iter.hasNext() && iter.next()>newValue1) {}
        iter.add(newValue1);
        while(iter.hasNext() && iter.next()>newValue2) {}
        iter.add(newValue2);
    }

    // read descriptors (matrix sizes etc)
    public static int[] parseIntArray(String line) {
        String[] parts = line.trim().split(" +");
        int[] result = new int[parts.length];
        for(int i=0;i<parts.length;i++) {
            result[i] = Integer.parseInt(parts[i]);
        }
        return result;
    }

    public static void main(String[] args) throws IOException {
        InputStream in;
        PrintStream out;
        if (args.length>0) {
            in = new FileInputStream(args[0]);
        } else {
            in = System.in;
        }
        if (args.length>1) {
            out = new PrintStream(new FileOutputStream(args[1]));
        } else {
            out = System.out;
        }
        new Solver(in, out).solveAll();
    }
}
