import java.io.*;
import java.util.*;

/**
 * Template to read multiple tasks and solve them
 */
public class Solver {

    private class Task {
        private final int chainCount;
        private final String[] chains;

        public Task(int chainCount, String[] chains) {
            this.chainCount = chainCount;
            this.chains = chains;
        }
    }

    private class Solution {

        private final long i;

        public Solution(long i) {

            this.i = i;
        }

        @Override
        public String toString() {
            return Long.toString(i);
        }
    }

    final InputStream in;
    final PrintStream out;

    public Solver(InputStream in, PrintStream out) {
        this.in = in;
        this.out = out;
    }

    /**
     * Read input.
     * First line is number of tasks.
     * Parse tasks and solve them.
     * @throws IOException if reading fails
     */
    public void solveAll() throws IOException {
        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        int cases = Integer.parseInt(reader.readLine());
        for(int i=0;i<cases;i++) {
            Task task = parseTask(reader);
            out.println("Case #" + (i + 1) + ": " + solve(task).toString());
        }
    }

    private Task parseTask(BufferedReader reader) throws IOException {
        int chainCount = Integer.parseInt(reader.readLine());
        String chains[] = parseStringArray(reader.readLine());
        return new Task(chainCount, chains);
    }

    private static final long MODULO = 1000000007L;

    private Solution solve(Task task) {
        Task optimized = compactStrings(task);
//        System.out.println(optimized.chainCount);
//        System.out.println(Arrays.toString(optimized.chains));
        if (!validTask(optimized)) {
//            System.out.println("Invalid");
            return new Solution(0);
        }
        if (optimized.chainCount==1) return new Solution(1);
        return new Solution(solveSolvable0(Arrays.asList(optimized.chains)));
    }

    private Task compactStrings(Task task) {
        List<String> result = new ArrayList<String>();
        for(String chain : task.chains) {
            result.add(compactString(chain));
        }
        return new Task(task.chainCount, result.toArray(new String[task.chainCount]));
    }

    private String compactString(String chain) {
        StringBuilder b = new StringBuilder();
        char last = 0;
        for(char c : chain.toCharArray()) {
            if (c!=last) {
                last = c;
                b.append(c);
            }
        }
        return b.toString();
    }

    private long solveSolvable0(List<String> available) {
        long result = 0;
        int fullSet[] = new int[26];
        Arrays.fill(fullSet, 1);
        for(String train : available) {
            List<String> remaining = new ArrayList<String>(available);
            remaining.remove(train);
            int availableNext[] = update(fullSet, train);
            result = (result + solveRemaining(train.charAt(train.length() - 1), availableNext, remaining)) % MODULO;
        }
        return result;
    }

    public static int[] update(int current[], String added) {
        int remChars[] = new int[26];
        System.arraycopy(current, 0, remChars, 0, 26);
        // all chars from string banned
        for(int i=0;i<added.length();i++) {
            char c = added.charAt(i);
            remove(c, remChars);
        }
        return remChars;
    }

    public static boolean canAdd(String toAdd, char last, int available[]) {
        int i = 0;
        for(;i<toAdd.length() && toAdd.charAt(i)==last;i++); // skip beginning if it matches last of prev string
        for(;i<toAdd.length();i++) {
            char c = toAdd.charAt(i);
            if (!allowed(c, available)) return false;
        }
        return true;
    }

    public static boolean allowed(char c, int allowed[]) {
        return allowed[c-'a']==1;
    }

    public static void remove(char c, int allowed[]) {
        allowed[c-'a'] = 0;
    }

    private long solveRemaining(char last, int available[], List<String> remaining) {
        long result = 0;
        for(String train : remaining) {
            if (canAdd(train, last, available)) {
                if (remaining.size()==1) {
                    return 1;
                } else {
                    List<String> remainingNext = new ArrayList<String>(remaining);
                    remainingNext.remove(train);
                    int availableNext[] = update(available, train);
                    result = (result + solveRemaining(train.charAt(train.length() - 1), availableNext, remainingNext)) % MODULO;
                }
            }
        }
        return result;
    }

    private static boolean validTask(Task task) {
        for(String chain : task.chains) {
            if (!validString(chain)) return false;
        }
        return true;
    }

    private static boolean validString(String chain) {
        int fullSet[] = new int[26];
        Arrays.fill(fullSet, 1);
        char last = 0;
        for(char c : chain.toCharArray()) {
            if (c!=last) {
                last = c;
                if (!allowed(c, fullSet)) return false;
                remove(c, fullSet);
            }
        }
        return true;
    }

//    private boolean checkUnmatching(String[] chains) {
//        Set<Character> ends = new HashSet<Character>();
//        for(String chain : chains) {
//            Character first = chain.charAt(0);
//            if (ends.contains(first)) ends.remove(first); else ends.add(first);
//            Character last = chain.charAt(chain.length()-1);
//            if (ends.contains(last)) ends.remove(last); else ends.add(last);
//        }
//
//        return ends.size()!=2 && ends.size()!=0;
//    }

    public static String[] parseStringArray(String line) {
        return line.trim().split(" +");
    }

    public static void main(String[] args) throws IOException {
        InputStream in;
        PrintStream out;
        if (args.length>0) {
            in = new FileInputStream(args[0]);
        } else {
            in = System.in;
        }
        if (args.length>1) {
            out = new PrintStream(new FileOutputStream(args[1]));
        } else {
            out = System.out;
        }
        new Solver(in, out).solveAll();
    }
}
