package qualifier.cookieClicker;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class CookieSolver {
	
	private static final String SOLUTION_SUCCESS_MESSAGE 	= "The solution has been written to: ";
	private static final String SOLUTION_CASE_PREFIX		= "Case #";
	
	private static final String USAGE_ERROR 				= "Usage: solver intputFilename outputFilename";
	private static final String INVALID_INPUT_FILE_ERROR 	= "The file you supplied does not exist.";
	private static final String INVALID_OUTPUT_FILE_ERROR 	= "The output file you specified is invalid.";

	private static final String TEST_CASES_ERROR = "An error occurred while trying to solve test cases: \n";
	private static final String NO_TEST_CASES_ERROR = TEST_CASES_ERROR + "No test cases were provided.";

	
	public static void main( String[] args ) {
		// Print an error and do nothing if no filename was provided
		if ( !(args.length == 2) ) {
			System.err.println( USAGE_ERROR );
		} else {
			// Make sure the file exists. If not, there is no input to read.
			File inputFile = new File( args[0] );
			
			if ( inputFile.exists() ) {
				List<CookieTestCase> testCases = InputParser.getTestCases( inputFile );
				try {
					// For this problem, the solutions are a simple text string that should be written to the output file
					List<String> solutions = solveTestCases( testCases );
					if ( writeSolutionsToOutputFile( solutions, args[1] ) ) {
						System.out.println( SOLUTION_SUCCESS_MESSAGE + args[1] );
					}
				} catch ( IllegalArgumentException e ) {
					System.err.println( e.getMessage() );
				} catch ( IOException e ) {
					System.err.println( e.getMessage() );
				}
			} else {
				System.err.println( INVALID_INPUT_FILE_ERROR );
			}
		}
	}
	
	/**
	 * Given a list of test cases, create a list of strings representing their solutions
	 * @param testCases						The test cases to solve
	 * @return								List<String> the solutions to the test cases
	 * @throws IllegalArgumentException		If no test cases are provided
	 */
	private static List<String> solveTestCases(List<CookieTestCase> testCases) throws IllegalArgumentException {
		// There must be test cases to solve
		if ( !(testCases.size() >= 1) ) {
			throw new IllegalArgumentException( NO_TEST_CASES_ERROR );
		}
		
		List<String> solutions =  new ArrayList<String>();
		
		int testCount = 1;
		for ( CookieTestCase testCase : testCases ) {
			// Get the contents of the test case (implementation-specific)
			String optimalTime = testCase.getOptimalCookieTime().toString();
			
			// Build the full solution string and add it to the list of solutions
			StringBuilder solutionBuilder = new StringBuilder();
			solutionBuilder.append( SOLUTION_CASE_PREFIX ).append( testCount++ );
			solutionBuilder.append( ": " ).append( optimalTime );
			
			solutions.add( solutionBuilder.toString() );
		}
		return solutions;
	}
	
	/**
	 * Given a list of solution strings, write those strings to an output file.
	 * @param solutions			The list of solution strings to write
	 * @param outputFilepath	The path to the file the solutions should be written to
	 * @return					true if the solutions were successfully written, false otherwise
	 * @throws IOException		If the solutions couldn't be written to the file
	 */
	
	private static boolean writeSolutionsToOutputFile(List<String> solutions, String outputFilepath) throws IOException {
		boolean writeSuccess = false;
		
		// Create the output file if it does not exist.
		File outputFile = new File( outputFilepath );
		if ( !outputFile.exists() ) {
			outputFile.getParentFile().mkdirs();
			try {
				outputFile.createNewFile();
			} catch (IOException e) {
				throw new IllegalArgumentException( INVALID_OUTPUT_FILE_ERROR );
			}
		}
		
		// Write the solutions to the output file
		BufferedWriter writer = null;
		try {
			writer = new BufferedWriter( new FileWriter( outputFile.getAbsoluteFile() ) );
			for ( String solution : solutions ) {
				writer.write( solution );
				writer.newLine();
			}
			
			writeSuccess = true;
		} finally {
			if ( writer != null ) {
				writer.close();
			}
		}
		
		return writeSuccess;
	}
}
