package qualifier.cookieClicker;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

public class CookieTestCase {
	private final BigDecimal 	farmCost;
	private final BigDecimal 	farmProduction;
	private final BigDecimal 	goalCookieNumber;
	
	private List<BigDecimal> 	farmPurchaseTimes;
	private BigDecimal 			optimalCookieTime = null;
	
	public CookieTestCase( BigDecimal farmCost, BigDecimal farmProduction, BigDecimal goalCookieNumber ) {
		this.farmCost = farmCost;
		this.farmProduction = farmProduction;
		this.goalCookieNumber = goalCookieNumber;
		farmPurchaseTimes = new ArrayList<BigDecimal>();
		optimalCookieTime = calculateOptimalCookieTime();
	}
	
	/**
	 * @return		The optimal time it takes to get the number of cookies configured in the object
	 * 				  with the given settings
	 */
	public BigDecimal getOptimalCookieTime() {
		return optimalCookieTime;
	}
	
	/**
	 * Calculates the optimal time to get the goal cookie number by inductively checking the time it
	 *   takes to add one more farm and wait for the cookies. If the time it takes including the total farm
	 *   purchase time is less than the stored optimal time, update the stored optimal time and continue.
	 *   
	 *   If adding another farm and waiting will take longer than the previous optimal time, the final time
	 *   can not be further optimized, so we finalize the optimal time and return.
	 * @return The optimal time to get the class's goal cookie count
	 */
	private BigDecimal calculateOptimalCookieTime() {
		BigDecimal result = null;
		boolean optimalResultFound = false;
		
		// Base cookie production will always start at 2 cookies per second
		BigDecimal baseCookieProduction = new BigDecimal( 2 );
		
		// Initialize the optimal cookie time
		optimalCookieTime = goalCookieNumber.divide(baseCookieProduction, 13, RoundingMode.UP);
		
		while ( !optimalResultFound ) {
			// Find the time to purchase a new farm at the starting base production rate for this iteration
			BigDecimal timeToPurchaseFarm = farmCost.divide( baseCookieProduction, 13, RoundingMode.UP );
			farmPurchaseTimes.add( timeToPurchaseFarm );
			baseCookieProduction = baseCookieProduction.add( farmProduction );
			
			// Find the time to buy cookies after adding another farm
			BigDecimal timeToBuyCookies = goalCookieNumber.divide(baseCookieProduction, 13, RoundingMode.UP);
			BigDecimal timeToBuyAllFarms = new BigDecimal(0);
			for ( BigDecimal farmPurchaseTime : farmPurchaseTimes ) {
				timeToBuyAllFarms = timeToBuyAllFarms.add( farmPurchaseTime );
			}
			BigDecimal newPurchaseTime = timeToBuyAllFarms.add( timeToBuyCookies );
			
			
			// Check the new time against the previous optimum
			if ( newPurchaseTime.compareTo( optimalCookieTime ) < 0 ) {
				optimalCookieTime = newPurchaseTime;
			} else {
				result = optimalCookieTime;
				optimalResultFound = true;
			}
		} // End while
		
		// Round the final result to 7 places
		return result.setScale(7, BigDecimal.ROUND_HALF_UP);
	}
}
