package qualifier.cookieClicker;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

public class InputParser {

	private static final String NON_NUMERIC_INPUT_ERROR = "The input file contains non-numeral characters.";
	private static final String READER_ERROR = "The file you provided could not be read.";

	/**
	 * Given an input file, parse its contents to create test cases
	 * @param inputFile		The file to read and parse
	 * @return				A list of test cases generated from the input file
	 */
	public static List<CookieTestCase> getTestCases(File inputFile) {
		List<CookieTestCase> testCases = new ArrayList<CookieTestCase>();
		Integer numTestCases = null;
		
		try {
			BufferedReader reader = new BufferedReader( new FileReader( inputFile ) );
			// First line always contains the number of test cases
			numTestCases = Integer.parseInt( reader.readLine() );
			
			for ( int i = 0; i < numTestCases; i++ ) {
				CookieTestCase testCase = getTestCaseFromReader( reader );
				if ( testCase != null ) {
					testCases.add( testCase );
				}
			}
			
		} catch (FileNotFoundException e) {
			// This is checked by the solver, and should never occur
			System.err.println( e.getMessage() );
		} catch (NumberFormatException e) {
			System.err.println( NON_NUMERIC_INPUT_ERROR );
		} catch (IOException e) {
			System.err.println( READER_ERROR );
		}
		
		return testCases;
	}
	
	/**
	 * Get the next input line and create a test case from it
	 * @param reader					BufferedReader containing the input
	 * @return							CookieTestCase initialized with the arguments from the file
	 */
	private static CookieTestCase getTestCaseFromReader(BufferedReader reader) throws NumberFormatException, IOException {
		// Get the input settings from the file
		String[] argumentStrings = reader.readLine().split( " " );
		
		// Parse the strings into BigDecimals representing the arguments
		DecimalFormat df = new DecimalFormat();
		df.setParseBigDecimal(true);
		
		BigDecimal farmCost = null;
		BigDecimal farmProduction = null;
		BigDecimal goalCookies = null;
		try {
			farmCost = (BigDecimal) df.parseObject( argumentStrings[0] );
			farmProduction = (BigDecimal) df.parseObject( argumentStrings[1] );
			goalCookies = (BigDecimal) df.parseObject( argumentStrings[2] );
		} catch (ParseException e) {
			System.err.println( NON_NUMERIC_INPUT_ERROR );
		}
		
		// Create a test case with the supplied arguments
		return new CookieTestCase( farmCost, farmProduction, goalCookies );
	}
}
