package com.google.qualificationround;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class CombinationGenerator {

	private final int[] a;
	private final int n;
	private final int r;
	private BigInteger numLeft;
	private final BigInteger total;

	// ------------
	// Constructor
	// ------------

	public CombinationGenerator(int n, int r) {
		if (r > n) {
			throw new IllegalArgumentException();
		}
		if (n < 1) {
			throw new IllegalArgumentException();
		}
		this.n = n;
		this.r = r;
		a = new int[r];
		BigInteger nFact = getFactorial(n);
		BigInteger rFact = getFactorial(r);
		BigInteger nminusrFact = getFactorial(n - r);
		total = nFact.divide(rFact.multiply(nminusrFact));
		reset();
	}

	// ------
	// Reset
	// ------

	public void reset() {
		for (int i = 0; i < a.length; i++) {
			a[i] = i;
		}
		numLeft = new BigInteger(total.toString());
	}

	// ------------------------------------------------
	// Return number of combinations not yet generated
	// ------------------------------------------------

	public BigInteger getNumLeft() {
		return numLeft;
	}

	// -----------------------------
	// Are there more combinations?
	// -----------------------------

	public boolean hasMore() {
		return numLeft.compareTo(BigInteger.ZERO) == 1;
	}

	// ------------------------------------
	// Return total number of combinations
	// ------------------------------------

	public BigInteger getTotal() {
		return total;
	}

	// ------------------
	// Compute factorial
	// ------------------

	private static BigInteger getFactorial(int n) {
		BigInteger fact = BigInteger.ONE;
		for (int i = n; i > 1; i--) {
			fact = fact.multiply(new BigInteger(Integer.toString(i)));
		}
		return fact;
	}

	// --------------------------------------------------------
	// Generate next combination (algorithm from Rosen p. 286)
	// --------------------------------------------------------

	public int[] getNext() {

		if (numLeft.equals(total)) {
			numLeft = numLeft.subtract(BigInteger.ONE);
			return a;
		}

		int i = r - 1;
		while (a[i] == n - r + i) {
			i--;
		}
		a[i] = a[i] + 1;
		for (int j = i + 1; j < r; j++) {
			a[j] = a[i] + j - i;
		}

		numLeft = numLeft.subtract(BigInteger.ONE);
		return a;

	}

	public static List<SeansList> getSeansList(BigInteger[] elements,
			int combinationParameter) {
		int[] indices;
		CombinationGenerator x = new CombinationGenerator(elements.length,
				combinationParameter);
		StringBuffer combination;
		List<SeansList> list = new ArrayList<SeansList>();
		while (x.hasMore()) {
			SeansList seansList = new SeansList();
			indices = x.getNext();
			for (int i = 0; i < indices.length; i++) {
				seansList.add(elements[indices[i]]);
			}
			list.add(seansList);
		}

		Collections.sort(list, new Comparator<SeansList>() {

			@Override
			public int compare(SeansList o1, SeansList o2) {
				if (o1.getSum().compareTo(o2.getSum()) > 0) {
					return -1;
				}

				if (o1.getSum().compareTo(o2.getSum()) < 0) {
					return 1;
				}
				return 0;

			}

		});

		return list;
	}

	public static void main(String[] args) {
		int[] elements = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15 };
		int[] indices;
		CombinationGenerator x = new CombinationGenerator(elements.length, 2);
		StringBuffer combination;
		while (x.hasMore()) {
			combination = new StringBuffer();
			indices = x.getNext();
			for (int i = 0; i < indices.length; i++) {
				combination.append(elements[indices[i]]);
				combination.append(",");
			}
			System.out.println(combination.toString());
		}

	}
}
