import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;


public class QDHallOfMirrors {
	
	private static double MAX_DISTANCE_FROM_BORDER_TO_CENTER = Math.sqrt(2)/2;

	private static class Room {
		public boolean[][] mirrors;
		
		public int maxLightDistance;
		
		public double startX;
		public double startY;
		
		public static Room readRoomData(BufferedReader reader) throws IOException {
			Room room = new Room();
			String line = reader.readLine();
			StringTokenizer tokenizer = new StringTokenizer(line);
			int roomHeight = Integer.parseInt(tokenizer.nextToken());
			int roomWidth = Integer.parseInt(tokenizer.nextToken());
			room.maxLightDistance = Integer.parseInt(tokenizer.nextToken());
			room.mirrors = new boolean[roomHeight][roomWidth];
			
			for (int i=0; i < roomHeight; i++) {
				line = reader.readLine();
				for (int j=0; j < roomWidth; j++) {
					switch (line.charAt(j)) {
					case '#': 
						room.mirrors[i][j] = true;
						break;
					case 'X':
						room.startX = j + 0.5;
						room.startY = i + 0.5;
					}
				}
			}
			
			return room;
		}
	}
	
	private static class Photon implements Cloneable {
		public double x;
		public double y;
		
		public double slopeY;
		public double slopeX;
		
		public double force;
		
		public int directionX;
		public int directionY;
		
		// indicates whether this photon is currently on border(s) of a cell
		public boolean atBorderX;
		public boolean atBorderY;
		
		public static Photon createNewPhoton(Room room) {
			Photon photon = new Photon();
			photon.x = room.startX;
			photon.y = room.startY;
			photon.force = room.maxLightDistance;
			
			return photon;
		}
		
		public void reverseDirection() {
			directionX = -directionX;
			directionY = -directionY;
		}
		
		public void moveToNextBorder() {
			double targetX = x;
			double targetY = y;
			double resultingX, resultingY;
			
			if (atBorderX) {
				targetX = x + directionX;
			} else if (directionX < 0) {
				targetX = Math.floor(x);
			} else if (directionX > 0) {
				targetX = Math.ceil(x);
			}
			
			if (atBorderY) {
				targetY = y + directionY;
			} else if (directionY < 0) {
				targetY = Math.floor(y);
			} else if (directionY > 0) {
				targetY = Math.ceil(y);
			} 
			
			if (slopeX == 0) {
				// special case, going vertically
				force -= Math.abs(y - targetY);
				y = targetY;
				atBorderY = true;
				return;
			}
			
			if (slopeY == 0) {
				// special case, going horizontally
				force -= Math.abs(x - targetX);
				x = targetX;
				atBorderX = true;
				return;
			}
			
			resultingY = slopeY*(targetX - x)/slopeX + y;
			resultingX = slopeX*(targetY - y)/slopeY + x;
			
			if ((directionX < 0 && resultingX < targetX) || (directionX > 0 && resultingX > targetX)) {
				force -= distance(x, y, targetX, resultingY);
				x = targetX;
				y = resultingY;
				atBorderX = true;
				atBorderY = Math.floor(y) == y;
			} else {
				force -= distance(x, y, resultingX, targetY);
				x = resultingX;
				y = targetY;
				atBorderX = Math.floor(x) == x;
				atBorderY = true;
			}
		}
		
		public void checkForMirrors(boolean[][] mirrors) {
			if (atBorderX && atBorderY) {
				// special case, we are in the corner
				boolean horizontalMirror, verticalMirror, diagonalMirror;
				int roomX = (int) x;
				int roomY = (int) y;
				if (directionX < 0 && directionY < 0) {
					horizontalMirror = mirrors[roomY][roomX-1];
					verticalMirror = mirrors[roomY-1][roomX];
					diagonalMirror = mirrors[roomY-1][roomX-1];
				} else if (directionX < 0 && directionY > 0) {
					horizontalMirror = mirrors[roomY-1][roomX-1];
					verticalMirror = mirrors[roomY][roomX];
					diagonalMirror = mirrors[roomY][roomX-1];
				} else if (directionX > 0 && directionY < 0) {
					horizontalMirror = mirrors[roomY][roomX];
					verticalMirror = mirrors[roomY-1][roomX-1];
					diagonalMirror = mirrors[roomY-1][roomX];
				} else {
					horizontalMirror = mirrors[roomY-1][roomX];
					verticalMirror = mirrors[roomY][roomX-1];
					diagonalMirror = mirrors[roomY][roomX];
				}
				
				if (horizontalMirror && diagonalMirror && verticalMirror) {
					reverseDirection();
				} else if (horizontalMirror && diagonalMirror) {
					reflectX();
				} else if (verticalMirror && diagonalMirror) {
					reflectY();
				} else if (diagonalMirror) {
					force = 0;
				}
				
			} else if (atBorderX) {
				// if there is a mirror in the next cell
				if (directionX < 0 && mirrors[(int) Math.floor(y)][((int) x) - 1] ||
						directionX > 0 && mirrors[(int) Math.floor(y)][(int) x] ) {
					reflectX();
				}
			} else if (atBorderY) {
				// if there is a mirror in the next cell
				if (directionY < 0 && mirrors[((int) y) - 1][(int) Math.floor(x)] ||
						directionY > 0 && mirrors[(int) y][(int) Math.floor(x)] ) {
					reflectY();
				}
			}
		}
		
		public boolean reachesStartPoint(Room room) {
			//if we are going in right direction
			if (Math.signum(room.startX - x) == directionX && Math.signum(room.startY - y) == directionY
					// and start point lies on our route
					&& (Math.abs(slopeX*(room.startY - y) - slopeY*(room.startX - x)) < 0.0000000000000006)) {
				
				double distance = distance(x, y, room.startX, room.startY);
				if (distance <= MAX_DISTANCE_FROM_BORDER_TO_CENTER && distance <= force) {
//					System.out.println(slopeX*(room.startY - y) - slopeY*(room.startX - x));
					return true;
				}
			}
			return false;
		}

		private void reflectX() {
			directionX = -directionX;
			slopeX = -slopeX;
		}
		
		private void reflectY() {
			directionY = -directionY;
			slopeY = -slopeY;
		}

		@Override
		public String toString() {
			StringBuilder sb = new StringBuilder();
			sb.append("pos(").append(x).append(", ").append(y).append(") ");
			sb.append("dir(").append(directionX).append(", ").append(directionY).append(") ");
			sb.append("slope(").append(slopeX).append(", ").append(slopeY).append(") ");
			sb.append("brdr(").append(atBorderX).append(", ").append(atBorderY).append(") ");
			sb.append("force: ").append(force);
			
			return sb.toString();
		}

		@Override
		public Photon clone() {
			Photon clone = new Photon();
			clone.x = x;
			clone.y = y;
			clone.slopeX = slopeX;
			clone.slopeY = slopeY;
			clone.force = force;
			clone.directionX = directionX;
			clone.directionY = directionY;
			clone.atBorderX = atBorderX;
			clone.atBorderY = atBorderY;
			return clone;
		}
	}
	
	public static double distance(double x1, double y1, double x2, double y2) {
		final double x = Math.abs(x1 - x2);
		final double y = Math.abs(y1 - y2);
		return Math.sqrt(x*x + y*y);
	}
	
	
	public static void solve(BufferedReader reader, BufferedWriter writer) throws IOException {
		int testCount = readIntFromLine(reader);
		
		for (int test = 1; test <= testCount; test++) {
			writer.append("Case #").append(Integer.toString(test)).append(": ");
			
			solveTestCase(reader, writer);
			
			writer.newLine();
		}
		writer.flush();
	}

	private static void solveTestCase(BufferedReader reader, BufferedWriter writer) throws IOException {
		Room room = Room.readRoomData(reader);
		
		List<Photon> photons = sendPhotonsInAllDirections(room);
		System.out.println("total photons: " + photons.size());
		int reflectionCount = 0;
		
		for (Photon photon : photons) {
//			System.out.println("new photon:");
//			System.out.println("  " + photon.toString());
			while (photon.force >= 0.5) {
				photon.moveToNextBorder();
//				System.out.println("st " + photon.toString());
				photon.checkForMirrors(room.mirrors);
//				System.out.println("rt " + photon.toString());
				if (photon.reachesStartPoint(room)) {
//					System.out.println("reached start point");
					reflectionCount++;
					break;
				}
			}
		}
		
		writer.append(Integer.toString(reflectionCount));
	}

	private static List<Photon> sendPhotonsInAllDirections(Room room) {
		List<Photon> photons = new ArrayList<Photon>();

		Set<Double> possibleSlopes = new HashSet<Double>();
		possibleSlopes.add((double) 0);
		
		for (int i=1; i <= room.maxLightDistance; i++) {
			double slope = -1;
//			for (int j=0; j < 2*i; j++) {
//				possibleSlopes.add(-1d + ((double)j)/i);
//			}
			while (slope < 1) {
				if (Math.abs(slope) > 0.001) {
					possibleSlopes.add(slope);
				}
				slope += 1.0d / i;
			}
		}
		
//		for (Double slope : possibleSlopes) {
//			System.out.println(slope);
//		}

		// create horizontally flying photons
		for (Double slope : possibleSlopes) {
			Photon photon = Photon.createNewPhoton(room);
			photon.slopeX = 1;
			photon.slopeY = slope;
			photon.directionX = -1;
			photon.directionY = - (int)Math.signum(slope);
			
			photons.add(photon);
			
			photon = photon.clone();
			photon.reverseDirection();
			photons.add(photon);
		}
		
		// create vertically flying photons
		for (Double slope : possibleSlopes) {
			Photon photon = Photon.createNewPhoton(room);
			photon.slopeX = slope;
			photon.slopeY = 1;
			photon.directionX = - (int)Math.signum(slope);
			photon.directionY = -1;
			
			photons.add(photon);
			
			photon = photon.clone();
			photon.reverseDirection();
			photons.add(photon);
		}
		return photons;
	}

	
	
	public static void solve(String inputFile, String outputFile) throws IOException {
		File input = new File(inputFile);
		File output = new File(outputFile);
		
		if (!input.exists()) {
			throw new IllegalArgumentException("Input file doesn't exist: " + inputFile);
		}
		
		BufferedReader reader = null;
		BufferedWriter writer = null;
		try {
			reader = new BufferedReader(new FileReader(input));
			writer = new BufferedWriter(new FileWriter(output));
			solve(reader, writer);
		
		} finally {
			if (reader != null)
				reader.close();
			if (writer != null)
				writer.close();
		}
	}
	
	public static int readIntFromLine(BufferedReader reader) throws IOException {
		String line = reader.readLine();
		int testCount = Integer.parseInt(line);
		return testCount;
	}
	
	public static void main(String[] args) throws IOException {
		System.out.println("Start...");
		solve("res/input.txt", "res/output.txt");
		System.out.println("Done!");
	}
}
