import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UncheckedIOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.OptionalInt;
import java.util.Scanner;
import java.util.function.Function;
import java.util.stream.Stream;

public class Harness {

    public static void main(String[] args) throws IOException {
        Harness.createGoogleHarness('B', true, false, 0, OptionalInt.of(1), true)
                .solve(Harness::solver);
    }

    static Solution solver(TestCase tc) {
        try (Scanner in = new Scanner(tc.asOneliner())) {
            int B = in.nextInt();
            int M = in.nextInt();
            if (B > M) {
                StringBuilder result = new StringBuilder();
                result.append("POSSIBLE" + "\r\n");
                
                String row = "";
                for(int i = 0; i < B-M; i++) {
                    row += "0";
                }
                for(int i = 0; i < M; i++) {
                    row += "1";
                }
                result.append(row).append("\r\n");
                
                for(int i = 0; i < B-M-1; i++) {
                    row = "";
                    for (int j = 0; j < B; j++) {
                        row += "0";
                    }
                    result.append(row).append("\r\n");
                }
                
                for(int i = 0; i < M-1; i++) {
                    row = "";
                    for (int j = 0; j < B-1; j++) {
                        row += "0";
                    }
                    row += "1";
                    result.append(row).append("\r\n");
                }
                
                row = "";
                for(int i = 0; i < B; i++) {
                    row += "0";
                }
                result.append(row);
                return Solution.create(tc, result.toString());
            } else {
                return Solution.create(tc, "IMPOSSIBLE");
            }
        }
    }

    static Harness createGoogleHarness(char problem, boolean small, boolean practice, int attempt, OptionalInt nbLinesPerTestCase, boolean submit) {
        String filename = String.format("%c-%s-%s", problem, small ? "small" : "large", practice ? "practice" : "attempt" + attempt);
        return new Harness(filename + ".in", nbLinesPerTestCase, filename + ".out", "Case #%d: %s%n", submit);
    }

    private final String inputFilename;
    private final OptionalInt nbLinesPerTestCase;
    private final String outputFilename;
    private final String outputFormat;
    private final boolean submit;

    Harness(String inputFilename, OptionalInt nbLinesPerTestCase, String outputFilename, String outputFormat, boolean submit) {
        this.inputFilename = inputFilename;
        this.nbLinesPerTestCase = nbLinesPerTestCase;
        this.outputFilename = outputFilename;
        this.outputFormat = outputFormat;
        this.submit = submit;
    }

    void solve(Function<TestCase, Solution> solver) throws IOException {
        try (OutputStream raw = getOutputStream();
                OutputStreamWriter rawWriter = new OutputStreamWriter(raw, StandardCharsets.UTF_8);
                BufferedWriter out = new BufferedWriter(rawWriter)) {
            readTestCases()
                    .parallel()
                    .map(solver)
                    .forEachOrdered(s -> {
                        try {
                            out.write(String.format(outputFormat, s.getTestCase().index, s.getFormattedSolution()));
                        } catch (IOException e) {
                            throw new UncheckedIOException(e);
                        }
                    });
        } catch (UncheckedIOException e) {
            throw e.getCause();
        }
    }

    private OutputStream getOutputStream() throws IOException {
        if (submit) {
            URL inputFile = Objects.requireNonNull(Harness.class.getResource(inputFilename), "Input file not found");
            try {
                Path outputFile = Paths.get(inputFile.toURI()).getParent().resolve(outputFilename);
                return Files.newOutputStream(outputFile);
            } catch (URISyntaxException e) {
                throw new IOException(e);
            }
        } else {
            return System.out;
        }
    }

    private Stream<TestCase> readTestCases() throws IOException {
        Stream.Builder<TestCase> resultBuilder = Stream.builder();
        InputStream raw = Objects.requireNonNull(Harness.class.getResourceAsStream(inputFilename), "Input file not found");
        try (InputStreamReader rawReader = new InputStreamReader(raw, StandardCharsets.UTF_8);
                BufferedReader in = new BufferedReader(rawReader)) {
            int nbTestCases = Integer.parseInt(in.readLine());
            for (int i = 0; i < nbTestCases; i++) {
                List<String> lines = new ArrayList<>();
                for (int j = 0; j < getTestCaseSize(in); j++) {
                    lines.add(in.readLine());
                }
                resultBuilder.accept(new TestCase(i + 1, lines));
            }
        }
        return resultBuilder.build();
    }

    private int getTestCaseSize(BufferedReader in) throws IOException {
        if (nbLinesPerTestCase.isPresent()) {
            return nbLinesPerTestCase.getAsInt();
        } else {
            return Integer.parseInt(in.readLine());
        }
    }

    static class TestCase {

        private final int index;
        private final List<String> lines;

        private TestCase(int index, List<String> lines) {
            this.index = index;
            this.lines = Collections.unmodifiableList(lines);

            if (index < 1) {
                throw new IllegalArgumentException("Illegal index");
            } else if (lines.isEmpty()) {
                throw new IllegalArgumentException("Empty test case");
            }
        }

        List<String> getLines() {
            return lines;
        }

        String asOneliner() {
            if (lines.size() == 1) {
                return lines.get(0);
            } else {
                throw new IllegalArgumentException("Not a oneliner");
            }
        }

    }

    static interface Solution {

        static Solution create(TestCase testCase, String formattedSolution) {
            return new Solution() {

                @Override
                public TestCase getTestCase() {
                    return testCase;
                }

                @Override
                public String getFormattedSolution() {
                    return formattedSolution;
                }

            };
        }

        TestCase getTestCase();

        String getFormattedSolution();

    }

}
