import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.HashMap;


public class WatershedsMap {
	private final static int NORTH = 0;
	private final static int WEST = 1;
	private final static int EAST = 2;
	private final static int SOUTH = 3;
	private int[] diffs;
	private HashMap<WatershedsCell, Character> sinks;
	private WatershedsCell[] cells;
	private boolean[] visited;
	private int height;
	private int width;
	private char c;

	public WatershedsMap(BufferedReader reader, int height, int width) throws IOException {
		cells = new WatershedsCell[height*width];
		visited = new boolean[height*width];
		sinks = new HashMap<WatershedsCell, Character>();
		diffs = new int[] { -width, -1, +1, +width };
		this.height = height;
		this.width = width;
		c = 'a';

		// Get altitudes for all cells.
		int i = 0;
		for (int h = 0; h < height; h ++) {
			String line = reader.readLine();
			String[] data = line.split(" ");
			for (int w = 0; w < width; w ++) {
				cells[i ++] = new WatershedsCell(Integer.parseInt(data[w]));
			}
		}

		// Trace streams.
		for (int y = 0; y < height; y ++) {
			for (int x = 0; x < width; x ++) {
				int y_ = y, x_ = x;
				while (true) {

					// Check if visited in a previous loop.
					int index = getIndex(x_, y_);
					if (visited[index]) {
						break;
					}
					visited[index] = true;

					// Look for the destination cell.
					final WatershedsCell cell = cells[index];
					int destDirection = -1;
					int minimumAltitude = cell.getAltitude();
					for (int d = 0; d < 4; d ++) {
						int neighborAltitude = getNeighborAltitude(x_, y_, d);
						if (neighborAltitude >= 0 &&
								neighborAltitude < minimumAltitude) {
							destDirection = d;
							minimumAltitude = neighborAltitude;
						}
					}

					// This cell is a sink.
					if (destDirection == -1) {
						sinks.put(cell, c ++);
						break;
					}

					// Water flow goes down to the destination cell.
					int destIndex = getNeighborIndex(x_, y_, destDirection);
					cell.setDestination(cells[destIndex]);

					// Continue loop with a new position.
					x_ = destIndex % width;
					y_ = destIndex / width;
				}
			}
		}
	}

	private int getIndex(int x, int y) {
		return y*width + x;
	}

	private int getNeighborIndex(int x, int y, int direction) {
		if (direction == NORTH && y <= 0 ||
				direction == WEST && x <= 0 ||
				direction == EAST && x >= width-1 ||
				direction == SOUTH && y >= height-1) {
			return -1;
		}
		return getIndex(x, y) + diffs[direction];
	}

	private int getNeighborAltitude(int x, int y, int direction) {
		int i = getNeighborIndex(x, y, direction);
		if (i < 0) { return -1; }
		return cells[i].getAltitude();
	}

	public void write(BufferedWriter writer) throws IOException {
		int i = 0;
		for (int h = 0; h < height; h ++) {
			for (int w = 0; w < width; w ++) {
				final WatershedsCell cell = cells[i ++];
				writer.write(
						sinks.get(cell.getFinalDestination()));
				if (w < width-1) {
					writer.write(' ');
				}
			}
			writer.newLine();
		}
	}
}
