package com.archerintel.codejam;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Scanner;

public class MinesweeperMaster {

	private static final String SMALL_IN_FILE_NAME = "C-small-attempt2.in";
	private static final String SMALL_OUT_FILE_NAME = "C-small-attempt2.out";
	private static final Charset ENCODING = StandardCharsets.ISO_8859_1;
	
	private static final String FAIL_MESSAGE = "Impossible";
	
	public static boolean isOnBoard(char board[][], int row, int col) {
		return (0 <= row) && (row < board.length)
				&& (0 <= col) && (col < board[0].length);
	}
	
	public static List<Integer> getNonEmptyNeighbors(char grid[][], int row,
			                                         int col) {
		List<Integer> result = new ArrayList<>();
		int neighborRow;
		int neighborCol;
		int numCols = grid[0].length;
		
		// Check over all cells within 1 Moore distance to the current cell:
		for (int i = -1; i <= 1; ++i) {
			for (int j = -1; j <= 1; ++j) {
				
				// Don't count the current cell:
				if ((i != 0) || (j != 0)) {
					neighborRow = row + i;
					neighborCol = col + j;
					
					// '.' signals an empty cell:
					if (isOnBoard(grid, neighborRow, neighborCol)
							&& (grid[neighborRow][neighborCol] != '.')) {
						result.add(neighborRow * numCols + neighborCol);
					}
				}
			}
		}
		
		return result;
	}

	public static void emptyCells(char grid[][], List<Integer> cells) {
		int numCols = grid[0].length;
		
		for (Integer cell : cells) {
			grid[cell / numCols][cell % numCols] = '.';
		}
	}
	
	// The grid was originally initialized with all zeros and filled in with
	// '.' for empty spaces and 'c' for the click space. All remaining spaces
	// with a 0 contain a mine:
	public static void printGridToWriter(PrintWriter writer, char grid[][]) {
		for (int i = 0; i < grid.length; ++i) {
			for (int j = 0; j < grid[0].length; ++j) {
				if (grid[i][j] != 0) {
					writer.print(grid[i][j]);
				} else {
					writer.print('*');
				}
			}
			writer.println();
		}
	}
	
	public static void main(String[] args) {
		Scanner inScan = null;
		PrintWriter writer = null;
		int numTestCases;
		int numRows;
		int numCols;
		int numCells;
		int numMines;
		int numEmpty;
		int currentEmpty = 0;
		int numVisited = 0;
		char grid[][] = null;
		boolean visited[][];
		List<Integer> peers;
		List<Integer> children;
		int clickCell = 0;
		Integer currentCell;
		List<Integer> nonEmptyNeighbors;
		int row;
		int col;
		
		
		// Set up the input and output:
		java.io.File unRead = new java.io.File(SMALL_IN_FILE_NAME);
		try {
			inScan = new Scanner(unRead);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		
		try {
			writer = new PrintWriter(SMALL_OUT_FILE_NAME);
		} catch (FileNotFoundException fnfExc) {
			fnfExc.printStackTrace();
		}
		
		// Read the input:
		numTestCases = inScan.nextInt();
	
		for (int i = 0; i < numTestCases; ++i) {
			
			// Read each test case input:
			inScan.nextLine();
			numRows = inScan.nextInt();
			numCols = inScan.nextInt();
			numMines = inScan.nextInt();
			
			// Strategy:
			//    Repeat until all cells on the board have been visited
			//    or the number of empty cells == numEmpty:
			//
			//    1)Clear all peers, subVisited, and children.
			//      Start at some square on the board that is not marked as
			//      visited and make this square empty.
			//      This is the current cell.
			//      Mark it as a peer.
			//      Mark it as visited.
			//
			//    2)Repeat until all peers are nonExpandable or the number
			//      of empty cells == numEmpty:
			//        Choose a peer that is not marked nonExpandable and make
			//        it the current cell.
			//        If emptying all the current cell's neighbors would not
			//        lead to more empty spaces than numEmpty:
			//          Empty all its neighbors.
			//          Mark them as children.
			//          Mark the current cell as subVisited.
			//          If the current cell has non-subVisited peers:
			//            One of the peers is the new current cell.
			//          Otherwise:
			//            Clear peers.
			//            Promote all children to peers, and choose one to be
			//            the current cell.
			//            Clear children.
			//        Otherwise:
			//          Mark current cell as nonExpandable.
			
			numCells = numRows * numCols;
			numEmpty = numCells - numMines;
			
			grid = new char[numRows][numCols];
			
			numVisited = 0;
			int j = 0;
			int k = 0;
			clickCell = 0;
			while ((j < numRows) && (currentEmpty != numEmpty)) {
				
				// We have a new start cell so clear old work.
				// When the loop conditions fail, the grid will contain
				// whatever was written to it in the previous iteration:
				currentEmpty = 0;
				grid = new char[numRows][numCols];
				peers = new ArrayList<>();
				children = new ArrayList<>();
				
				// Mark the click cell as empty for now:
				grid[j][k] = '.';
				++currentEmpty;
				
				peers.add(clickCell);
				
				while (!peers.isEmpty() && (currentEmpty != numEmpty)) {
					currentCell = peers.get(0);
					row = currentCell / numCols;
					col = currentCell % numCols;
					
					nonEmptyNeighbors = getNonEmptyNeighbors(grid, row, col);
					if (!nonEmptyNeighbors.isEmpty()
							&& ((currentEmpty + nonEmptyNeighbors.size())
							    <= numEmpty)) {
						emptyCells(grid, nonEmptyNeighbors);
						children.addAll(nonEmptyNeighbors);
						currentEmpty += nonEmptyNeighbors.size();
						
						// Equivalent to marking the cell as subVisited:
						peers.remove(currentCell);
						
						// If there aren't any remaining non-subVisited peers:
						if (peers.isEmpty()) {
							peers.addAll(children);
							children.clear();
						}
					} else {
						
						// Equivalent to marking the cell nonExpandable:
						peers.remove(currentCell);
					}
					
				}
				
				
				// Move to the next cell in the grid. Each cell is visited
				// once (implicitly "marked" as visited):
				++k;
				if (k == numCols) {
					k = 0;
					++j;
				}
				
				// The next starting cell is at grid[j][k]:
				// Mark it as the "click" cell:
				if (currentEmpty != numEmpty) {
					clickCell = j * numCols + k;
				}
				
			}
			
			// Print the results:
			writer.println("Case #" + (i + 1) + ":");
			
			if (currentEmpty == numEmpty) {
				
				// Success!
				grid[clickCell / numCols][clickCell % numCols] = 'c';
				printGridToWriter(writer, grid);
			} else {
				writer.println(FAIL_MESSAGE);
			}
			
		}
		
		writer.close();
	}

}
