package jam;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.PrintStream;
import static java.lang.System.currentTimeMillis;
import java.util.*;

/**
 * User: looser
 * Date: 04.09.2009
 */
public class Watersheds {
    private final int width;
    private final int height;
    private final Cell[][] m;
    private final Namer namer = new Namer();

    public Watersheds(int width, int height, int[][] m) {
        this.width = width;
        this.height = height;
        this.m = new Cell[height][];

        for (int h = 0; h < height; h++) {
            this.m[h] = new Cell[width];
            for (int w = 0; w < width; w++) {
                this.m[h][w] = new Cell(m[h][w], w, h);
            }
        }
    }

    private void process() {
        processCells();
        nameBasins();
    }

    private void processCells() {
        for (int h = 0; h < height; h++) {
            for (int w = 0; w < width; w++) {
                Cell current = m[h][w];
                Cell target = whereLeaks(h, w);

                if (target != null) {
                    current.setDownwardsLink(target);
                }
            }
        }
    }

    private Cell whereLeaks(int h, int w) {
        Cell current = m[h][w];
        List<Cell> neib = new ArrayList<Cell>(4);
        if (h > 0) { // North
            neib.add(m[h-1][w]);
        }
        if (w > 0) { // West
            neib.add(m[h][w-1]);
        }
        if (w < width-1) { // East
            neib.add(m[h][w+1]);
        }
        if (h < height-1) { // South
            neib.add(m[h+1][w]);
        }

        int minAlt = current.alt;
        for (Cell cell : neib) {
            minAlt = cell.alt < minAlt ? cell.alt : minAlt;
        }

        if (minAlt == current.alt) return null;

        for (Cell cell : neib) {
            if (minAlt == cell.alt) {
                return cell;
            }
        }

        throw new RuntimeException("Never gonna happen");
    }

    private void nameBasins() {
        for (int h = 0; h < height; h++) {
            for (int w = 0; w < width; w++) {
                Cell current = m[h][w];
                if (current.isNamed()) {
                    continue;
                }

                Cell sink = current.getSink();
                sink.setName(namer.next());
            }
        }
    }

    @Override
    public String toString() {
        StringBuilder s = new StringBuilder();
        for (int h = 0; h < height; h++) {
            for (int w = 0; w < width; w++) {
                Cell current = m[h][w];
                s.append(current.name()).append(" ");
            }
            s.append("\n");
        }
        return s.toString();
    }

    private class Cell {
        public final int alt;
        public final int w;
        public final int h;

        private Cell downwards = null;
        private Set<Cell> upwards = new HashSet<Cell>();

        private String name = null;

        public Cell(int alt, int w, int h) {
            this.alt = alt;
            this.w = w;
            this.h = h;
        }

        public boolean isNamed() {
            return name != null;
        }

        public String name() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
            for (Cell cell : upwards) {
                cell.setName(name);
            }
        }

        public void setDownwardsLink(Cell target) {
            downwards = target;
            target.upwards.add(this);
        }

        public Cell getSink() {
            if (downwards == null) return this;
            return downwards.getSink();
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            Cell cell = (Cell) o;

            if (h != cell.h) return false;
            if (w != cell.w) return false;

            return true;
        }

        @Override
        public int hashCode() {
            int result = w;
            result = 31 * result + h;
            return result;
        }
    }

    private static class Namer {
        private char current = 'a';

        public String next() {
            String res = String.valueOf(current);
            current++;
            return res;
        }
    }

    public static void main(String[] args) throws Exception {
        BufferedReader f = new BufferedReader(new FileReader("input2.txt"));
        //PrintStream out = System.out;
        PrintStream out = new PrintStream("output2.txt");

        long start = currentTimeMillis();
        int n = Integer.parseInt(f.readLine());
        for (int i = 0; i < n; i++) {
            String[] s = f.readLine().split(" ");
            int h = Integer.parseInt(s[0]);
            int w = Integer.parseInt(s[1]);
            int [][] m = new int[h][];

            for (int j = 0; j < h; j++) {
                s = f.readLine().split(" ");
                m[j] = new int[w];
                for (int k = 0; k < w; k++) {
                    m[j][k] = Integer.parseInt(s[k]);
                }
            }


            Watersheds sheds = new Watersheds(w, h, m);
            sheds.process();
            out.print("Case #" + (i+1) + ":\n" + sheds);
        }

        System.out.println("Total time: " + (currentTimeMillis() - start));

    }
}
