package codejam.util;

public class DefaultMatrix<T> implements Matrix<T> {

    private final int cols, rows;
    private final T[] values;
    private final Integer[] directions;

    public DefaultMatrix(int cols, int rows, T[] values) {
        if (!((cols * rows) == values.length)) {
            throw new IllegalArgumentException("Values must match the given matrix size");
        }

        this.cols = cols;
        this.rows = rows;
        this.values = values;

        directions = new Integer[]{
            NORTHWEST, NORTHEAST, SOUTHWEST, SOUTHEAST, NORTH, EAST, SOUTH, WEST
        };
    }
    
    @Override
    public int cols() {
        return this.cols;
    }

    @Override
    public int rows() {
        return this.rows;
    }
    

    @Override
    public T valueAt(int col, int row) {
        if (!this.isValidColumn(col) || !this.isValidRow(row)) {
            throw new IllegalArgumentException("Column does not exist: " + col);
        }

        return this.values[this.indexFromColRow(col, row)];
    }

    @Override
    public T valueAt(int index) {
        if (!isValidIndex(index)) {
            throw new IllegalArgumentException("Index does not exist: " + index);
        }

        return this.values[index];
    }

    @Override
    public boolean hasNeighbour(int col, int row, int direction) {
        int moveAmount = calcIndexChangeFromDirection(direction, col);

        Integer newPos = this.indexFromColRow(col, row) + moveAmount;
        return isValidIndex(newPos);
    }

    @Override
    public T neighbour(int col, int row, int direction) {
        int moveAmount = calcIndexChangeFromDirection(direction, col);

        Integer newPos = this.indexFromColRow(col, row) + moveAmount;
        if (!isValidIndex(newPos)) {
            throw new IllegalArgumentException("Cannot move in direction " + direction + " from pos " + col + ", " + row);
        }
        return this.values[newPos];
    }
    
    @Override
    public int indexAfterMove(int col, int row, int direction) {
        int moveAmount = calcIndexChangeFromDirection(direction, col);

        Integer newPos = this.indexFromColRow(col, row) + moveAmount;
        if (!isValidIndex(newPos)) {
            throw new IllegalArgumentException("Cannot move in direction " + direction + " from pos " + col + ", " + row);
        }
        return newPos;
    }
    
    @Override
    public Point positionAfterMove(int col, int row, int direction) {
        int moveAmount = calcIndexChangeFromDirection(direction, col);

        Integer newPos = this.indexFromColRow(col, row) + moveAmount;
        if (!isValidIndex(newPos)) {
            throw new IllegalArgumentException("Cannot move in direction " + direction + " from pos " + col + ", " + row);
        }
        
        return new Point(newPos % this.cols, newPos / this.rows);
    }
    
    @Override
    public void setValueAt(int col, int row, T value) {
        this.values[indexFromColRow(col, row)] = value;
    }

    private boolean isValidColumn(int col) {
        return (col > -1 && col < this.cols);
    }

    private boolean isValidRow(int row) {
        return (row > -1 && row < this.rows);
    }

    private boolean isCorrectDirection(int direction) {
        for (Integer dir : this.directions) {
            if (dir == direction) {
                return true;
            }
        }
        return false;
    }

    private int indexFromColRow(int col, int row) {
        return (row * this.cols) + col;
    }

    private boolean isValidIndex(int index) {
        return !(index < 0 || index >= this.values.length);
    }

    private int calcIndexChangeFromDirection(int direction, int col) {
        if (!isCorrectDirection(direction)) {
            throw new IllegalArgumentException("That direction does not exist: " + direction);
        }
        
        int moveAmount = 0;
        switch (direction) {
            case NORTHWEST:
                moveAmount -= this.cols;
                if (col % this.cols == 0) {
                    moveAmount = Integer.MIN_VALUE;
                } else {
                    moveAmount -= 1;
                }
                break;
            case NORTHEAST:
                moveAmount -= this.cols;
                if ((col + 1) % this.cols == 0) {
                    moveAmount = Integer.MIN_VALUE;
                } else {
                    moveAmount += 1;
                }
                break;
            case SOUTHWEST:
                moveAmount += this.cols;
                if (col % this.cols == 0) {
                    moveAmount = Integer.MIN_VALUE;
                } else {
                    moveAmount -= 1;
                }
                break;
            case SOUTHEAST:
                moveAmount += this.cols;
                if ((col + 1) % this.cols == 0) {
                    moveAmount = Integer.MIN_VALUE;
                } else {
                    moveAmount += 1;
                }
                break;
            case NORTH:
                moveAmount -= this.cols;
                break;
            case SOUTH:
                moveAmount += this.cols;
                break;
            case WEST:
                if (col % this.cols == 0) {
                    moveAmount = Integer.MIN_VALUE;
                } else {
                    moveAmount -= 1;
                }
                break;
            case EAST:
                if ((col + 1) % this.cols == 0) {
                    moveAmount = Integer.MIN_VALUE;
                } else {
                    moveAmount += 1;
                }
                break;
        }
        return moveAmount;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (int row = 0; row < this.rows; row++) {
            for (int col = 0; col < this.cols; col++) {
                sb.append(this.valueAt(col, row)).append(" ");
            }
            if (row < this.rows - 1) {
                sb.append("\n");
            }
        }
        return sb.toString();
    }

}
