package QR2017;

/**
 * Project: google-code-jam
 * Author:  Brian Dang
 * Date:    4/7/2017
 */

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

/**
 *  Problem:
 *      - Pancake has happy side and blank side.
 *      - You are head cook.
 *      - Pancakes are cooked in a single row over a hot surface.
 *      - To maximize efficiency, you've been given an oversized pancake flipper that flips *exactly* K consecutive
 *      pancakes.
 *      - In that range of K pancakes, it changes every happy-side pancake to a blank-side and vice versa.
 *      - It does NOT change the left-to-right order of those pancakes.
 *
 *      - You can't flip fewer than K pancakes at a time even at the ends of the row. (Raised borders on both sides)
 *      - You can flip the first K pancakes, but not the first K-1 pancakes.
 *
 *      - Your apprentice cook used the old fashioned single-pancake flipper to flip some individual pancakes and then
 *      ran to the restroom with it, right before the time when customers come to visit the kitchen.
 *      - You only have the oversized pancake flipper left and need to use it quickly to leave all the cooking
 *      pancakes happy side up so the customers feel happy.
 *
 *      - Given the current state of pancakes, calculate the MINIMUM number of uses of the oversized pancake flipper
 *      needed to leave all pancakes happy side up or state that there is no way to do it.
 *
 *  Input:
 *      - First line of input gives number of test cases, T. T test cases follow.
 *      - Each consist of one line w/ a string S and an integer K. K is the # of pancakes it can flip.
 *      - S is the row of pancakes where each character is either + (happy) or - (blank side)
 *
 *  Output:
 *      - For each test case, output one line containing Case #x: y, where x is the test case number starting with
 *      1 and y is either IMPOSSIBLE if there i no way to get all the pancakes happy side up, or an integer representing
 *      the minimum number of times you need to use the oversized pancake flipper to do it.
 *
 *  Limits:
 *      - 1 <= T <= 100
 *      - Every character in S is either + or -
 *      - 2 <= K <= length of S
 *
 *  Small dataset:
 *      2 <= length of S <= 10
 *
 *  Large dataset:
 *      2 <= length of S <= 1000
 */
public class OversizedPancakeFlipper {

    /**
     * Determines if the pancakes are all on their happy side.
     * @param pancakes the pancakes input
     * @return true if all pancakes are on their happy side. otherwise, false.
     */
    public boolean areAllPancakesHappySide(String pancakes) {
        if(pancakes == null || pancakes.length() < 2) {
            return false;
        }

        int pancakesLength = pancakes.length();
        for(int i=0; i < pancakesLength; i++) {
            if(pancakes.charAt(i) != '+')
                return false;
        }
        return true;
    }

    /**
     * Determines if the start index and end index is in valid range of the pancakes.
     * @param startIdx the start index
     * @param endIdx the end index
     * @param pancakes the pancakes input
     * @return true if the start + end index are valid. otherwise, false.
     */
    public boolean isValidFlippingRange(int startIdx, int endIdx, String pancakes) {
        // System.out.println("isValidFlippingRange(" + startIdx + ", " + endIdx + ")");
        if(pancakes == null || pancakes.length() < 2)
            return false;

        int pancakeLength = pancakes.length();
        int lastPancakeIndex = pancakeLength - 1;
        if(startIdx < 0 || startIdx > lastPancakeIndex)
            return false;
        if(endIdx < 0 || endIdx > lastPancakeIndex)
            return false;
        if(startIdx > endIdx)
            return false;

        return true;
    }

    /**
     * Flips N number of pancakes, where N is the flipper size.
     * The start of the flipper will be positioned at the given starting index.
     * @param startingIndex the start index to flip
     * @param pancakes the input pancakes
     * @param flipperSize the # of pancakes the flipper can flip
     * @return the updated pancakes string
     */
    public String flipPancakesLeftToRight(int startingIndex, String pancakes, int flipperSize) {
        if(pancakes == null || pancakes.length() < 2)
            throw new IllegalArgumentException("Pancakes is null or pancakes size is less than 2!");
        if(flipperSize < 2 || flipperSize > pancakes.length())
            throw new IllegalArgumentException("Flipper Size is invalid (less than 2 or greater than pancakes length)!");
        if(startingIndex < 0)
            throw new IllegalArgumentException("Starting Index cannot be less than 0.");
        if(!isValidFlippingRange(startingIndex, startingIndex + flipperSize - 1, pancakes))
            throw new IllegalArgumentException("Invalid flipping range! [startIdx: " + startingIndex +
                    ", flipperSize: " + flipperSize + ", pancakes: " + pancakes);

        StringBuffer sb = new StringBuffer();
        for(int i=0; i < startingIndex; i++) {
            sb.append(pancakes.charAt(i));
        }

        for(int i=startingIndex; i < (startingIndex+flipperSize); i++) {
            char c = pancakes.charAt(i);
            if(c == '+') {
                sb.append('-');
            } else if(c == '-') {
                sb.append('+');
            }
        }

        int pancakeLength = pancakes.length();
        for(int i=(startingIndex+flipperSize); i < pancakeLength; i++) {
            sb.append(pancakes.charAt(i));
        }

        return sb.toString();
    }

    /**
     * Flips N number of pancakes, where N is the flipper size.
     * The end of the flipper will be positioned at the given end index.
     * @param endIdx the end index to flip
     * @param pancakes the input pancakes
     * @param flipperSize the # of pancakes the flipper can flip
     * @return the updated pancakes string
     */
    public String flipPancakesRightToLeft(int endIdx, String pancakes, int flipperSize) {
        if(pancakes == null || pancakes.length() < 2)
            throw new IllegalArgumentException("Pancakes is null or pancakes size is less than 2!");
        if(flipperSize < 2 || flipperSize > pancakes.length())
            throw new IllegalArgumentException("Flipper Size is invalid (less than 2 or greater than pancakes length)!");
        if(endIdx < 0 || endIdx >= pancakes.length())
            throw new IllegalArgumentException("End Index cannot be less than 0 or greater than last pancake index.");
        if(!isValidFlippingRange(endIdx - flipperSize + 1, endIdx, pancakes))
            throw new IllegalArgumentException("Invalid flipping range! [endIdx: " + endIdx+
                    ", flipperSize: " + flipperSize + ", pancakes: " + pancakes);

        int startIdx = endIdx - flipperSize + 1;
        StringBuffer sb = new StringBuffer();
        for(int i=0; i < startIdx; i++) {
            sb.append(pancakes.charAt(i));
        }

        for(int i=startIdx; i < (startIdx+flipperSize); i++) {
            char c = pancakes.charAt(i);
            if(c == '+')
                sb.append('-');
            else if(c == '-')
                sb.append('+');
        }

        int pancakeLength = pancakes.length();
        for(int i=(startIdx+flipperSize); i < pancakeLength; i++) {
            sb.append(pancakes.charAt(i));
        }
        return sb.toString();
    }

    /**
     * Given pancakes and a flipper size, this determines the # of minimum flips.
     * @param pancakes the pancakes input
     * @param flipperSize the size of the flipper (how many pancakes it can flip)
     * @return returns the # of minimum flips required for all pancakes to be happy side up.
     *         If it is not possible, this will return "IMPOSSIBLE"
     */
    public String getMinimumFlips(String pancakes, int flipperSize) {
        System.out.println("Pancakes: " + pancakes);
        if(pancakes == null || pancakes.length() < 2)
            throw new IllegalArgumentException("Pancakes is null or pancakes size is less than 2!");
        if(flipperSize < 2)
            throw new IllegalArgumentException("Flipper Size is invalid (less than 2)!");

        int pancakesLength = pancakes.length();
        int flips = 0;

        while(true) {
            List<Integer> blankPancakes = getBlankPancakes(pancakes);
            for(Integer blankIndex : blankPancakes) {
                // Check if the blank can be flipped from the left
                if(isValidFlippingRange(blankIndex, blankIndex + flipperSize - 1, pancakes)) {
                    pancakes = flipPancakesLeftToRight(blankIndex, pancakes, flipperSize);
                    flips++;
                    break;
                }
                // Check if the blank can be flipped from the right
                else if(isValidFlippingRange(blankIndex - flipperSize + 1, blankIndex, pancakes)) {
                    pancakes = flipPancakesRightToLeft(blankIndex, pancakes, flipperSize);
                    flips++;
                    break;
                }
                // If it can't be flipped as start index or end index, it's impossible
                else {
                    return "IMPOSSIBLE";
                }
            }

            System.out.println("Flips [" + flips + "]: " + pancakes);

            if(flips > pancakesLength*2) {
                return "IMPOSSIBLE";
            }

            // Check if all pancakes are happy after each flip
            if(areAllPancakesHappySide(pancakes)) {
                break;
            }
        }

        return String.valueOf(flips);
    }

    /**
     * Retrieves the indices of pancakes w/ blank sides up.
     * @param pancakes the pancakes String
     * @return returns the list of indices of pancakes w/ blank sides up.
     */
    public List<Integer> getBlankPancakes(String pancakes) {
        if(pancakes == null || pancakes.length() < 2)
            throw new IllegalArgumentException("Pancakes is null or pancakes size is less than 2!");

        List<Integer> result = new ArrayList<Integer>();
        int pancakesLength = pancakes.length();
        for(int i=0; i < pancakesLength; i++) {
            if(pancakes.charAt(i) == '-')
                result.add(i);
        }

        return result;
    }

    /**
     * Creates an output file w/ solutions to the test cases in the provided file.
     * @param filePath the input file w/ test cases.
     */
    public void runTestCases(String filePath) {
        try {
            PrintWriter pw = new PrintWriter(new File("solution-" + filePath));
            File input = new File(filePath);
            Scanner scan = new Scanner(input);
            int numTestCases = Integer.valueOf(scan.nextLine());
            for(int i=1; i <= numTestCases; i++) {
                String pancakes = scan.next();
                int flipperSize = scan.nextInt();
                String minimumFlips = getMinimumFlips(pancakes, flipperSize);
                System.out.println(pancakes + " => " + minimumFlips);
                pw.write("Case #" + i + ": " + minimumFlips + "\n");
            }

            scan.close();
            pw.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }
}
