package QR2017;

/**
 * Project: google-code-jam
 * Author:  Brian Dang
 * Date:    4/7/2017
 */

import org.junit.Test;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Scanner;

import static org.junit.Assert.assertEquals;

/**
 *  Problem:
 *      - A certain bathroom has N + 2 stalls in a single row.
 *      - Stalls on left and right ends are permanently occupied by the bathroom guards.
 *      - The other N stalls are for users.
 *
 *      - When someone enters the bathroom ,they try to choose a stall that is as far from other people as possible.
 *      - To avoid confusion, they follow deterministic rules:
 *          1) For each empty stall S, they compute two values Ls and Rs; each of which is the # of empty stalls
 *          between S and the closest occupied stall to the left or right.
 *          2) Then they consider the set of stalls w/ the "farthest closest neighbor" between the L and R; that is
 *          those S for which min(Ls, Rs) is maximal!
 *          3) If there is only one such stall, they choose it. Otherwise, they choose the one among those where max
 *          (Ls, Rs) is maximal.
 *          4) If there are still multiple tied stalls, they choose the leftmost stall among those.
 *
 *      - K people are about to enter the bathroom; each one will choose their stall before the next arrives.
 *      - Nobody will ever leave.
 *
 *      - When the last person chooses their stall S, what will the values of max(Ls, Rs) and min(Ls, Rs) be?
 *
 *  Input:
 *      - First line gives # of test cases, T. T lines will follow.
 *      - Each line describes a test case w/ 2 integers N and K.
 *
 *  Output:
 *      - For each test case, output one line containing Case #x: y z, where x is test case # starting from 1,
 *      y is max(Ls, Rs), and z is min(Ls, Rs) calculated by the last person to enter for their chosen stall S.
 *
 *  Limits
 *      1 <= T <= 100
 *      1 <= K <= N
 *
 *  Small Data Set 1:
 *      1 <= N <= 1000
 *
 *  Small Data Set 2:
 *      1 <= N <= 10^6
 *
 *  Large Data Set:
 *      1 <= N <= 10^18
 *
 *   Solution:
 *      - Consider this:
 *          L       S       R
 *          2               2
 *          3               4
 *          3               6
 *          6               3
 *
 *        The one with 3(L) and 6(R) is the winner.
 *      - Need to use data type 'long' to handle 10^18 values
 *      - Create a StallInfo class to perform Comparable ranking between stall index, left empty spaces, right empty spaces.
 *
 */
public class BathroomStalls {
    /**
     * StallInfo is a class representing a Stall given a Bathroom's state.
     * - Stall Index
     * - # of empty stalls to the left of it.
     * - # of empty stalls to the right of it.
     */
    public static class StallInfo implements Comparable<StallInfo>{
        private long stallIndex;
        private long leftEmpty;
        private long rightEmpty;

        public StallInfo(long stallIndex, long leftEmpty, long rightEmpty) {
            this.stallIndex = stallIndex;
            this.leftEmpty = leftEmpty;
            this.rightEmpty = rightEmpty;
        }

        public long getStallIndex() {
            return stallIndex;
        }

        public long getLeftEmpty() {
            return leftEmpty;
        }

        public long getRightEmpty() {
            return rightEmpty;
        }

        public int compareTo(StallInfo o) {
            long myMin = Math.min(leftEmpty, rightEmpty);
            long otherMin = Math.min(o.getLeftEmpty(), o.getRightEmpty());

            long myMax = Math.max(leftEmpty, rightEmpty);
            long otherMax = Math.max(o.getLeftEmpty(), o.getRightEmpty());

            // Sort largest Min as first
            if(myMin > otherMin) {
                return -1;
            } else if(myMin < otherMin) {
                return 1;
            }

            // If Min is tied, sort largest Max as first
            if(myMax > otherMax) {
                return -1;
            } else if(myMax < otherMax) {
                return 1;
            }

            // If Min and Max are tied, sort by the Left Most Stall (smaller index)
            if(stallIndex < o.getStallIndex()) {
                return -1;
            } else if(stallIndex > o.getStallIndex()) {
                return 1;
            }

            return 0;
        }

        @Override
        public String toString() {
            return "StallInfo{" +
                    "stallIndex=" + stallIndex +
                    ", leftEmpty=" + leftEmpty +
                    ", rightEmpty=" + rightEmpty +
                    '}';
        }
    }

    /**
     * Given a list of stalls and index of a stall, determines the # of empty stalls to the left of it.
     * @param stalls the List of stalls; where a Boolean true represents occupied. A Boolean false represents unoccupied.
     * @param index the stall's index
     * @return the # of empty stalls to the left of the given stall
     */
    public int getNumEmptyLeftStalls(List<Boolean> stalls, int index) {
        if(stalls == null || stalls.size() == 0)
            return 0;

        int length = stalls.size();
        if(index < 0 || index >= length)
            throw new IllegalArgumentException("Index cannot be less than 0 or greater than last index of stalls!");

        int numEmptyStalls = 0;
        for(int i=index-1; i >= 0; i--) {
            if(stalls.get(i) == false) {
                numEmptyStalls++;
            } else {
                break;
            }
        }
        return numEmptyStalls;
    }

    /**
     * Given a list of stalls and index of a stall, determines the # of empty stalls to the right of it.
     * @param stalls the List of stalls; where a Boolean true represents occupied. A Boolean false represents unoccupied.
     * @param index the stall's index
     * @return the # of empty stalls to the right of the given stall
     */
    public int getNumEmptyRightStalls(List<Boolean> stalls, int index) {
        if(stalls == null || stalls.size() == 0)
            return 0;

        int length = stalls.size();
        if(index < 0 || index >= length)
            throw new IllegalArgumentException("Index cannot be less than 0 or greater than last index of stalls!");

        int numEmptyStalls = 0;
        for(int i=index+1; i < length; i++) {
            if(stalls.get(i) == false) {
                numEmptyStalls++;
            } else {
                break;
            }
        }
        return numEmptyStalls;
    }

    /**
     * Given a bathroom w/ a number of stalls and number of people who sequentially select a stall,
     * this function determines which Stall the last Person selects and returns the max(Ls, Rs) and min(Ls, Rs);
     * where Ls is the number of empty left stalls and Rs is the number of empty right stalls.
     * @param numStalls the number of stalls in the bathroom
     * @param numPeople the number of people who enter the bathroom
     * @return the max(Ls, Rs) and min(Ls, Rs) of the stall selected by the last person
     */
    public String getMaxAndMinForLastPerson(int numStalls, int numPeople) {
        if(numStalls < 1)
            throw new IllegalArgumentException("Number of stalls can't be less than 1!");
        if(numPeople < 1 || numPeople > numStalls)
            throw new IllegalArgumentException("# of people can't be less than 1 or more than # of stalls!");

        // Initialize Stalls
        List<Boolean> stalls = new ArrayList<Boolean>();
        for(int i=0; i < numStalls; i++)
            stalls.add(false);

        StallInfo lastPersonStall = null;

        // Each person selects a stall
        for(int p=0; p < numPeople; p++) {
            List<StallInfo> tempStallInfo = new ArrayList<StallInfo>();
            for(int s=0; s < numStalls; s++) {
                // Iterate through each of the non-occupied stalls
                if(stalls.get(s) == false) {
                    int numEmptyLeftStalls = getNumEmptyLeftStalls(stalls, s);
                    int numEmptyRightStalls = getNumEmptyRightStalls(stalls, s);
                    StallInfo si = new StallInfo(s, numEmptyLeftStalls, numEmptyRightStalls);
                    tempStallInfo.add(si);
                }
            }

            // Determine which stall the person should go into. First stall is the chosen one
            Collections.sort(tempStallInfo);
            StallInfo selectedStall = tempStallInfo.get(0);

            // Mark this stall as occupied
            stalls.set((int) selectedStall.getStallIndex(), true);
            if(p == (numPeople-1)) {
                lastPersonStall = selectedStall;
            }
        }

        // Return the max(Ls, Rs) and min(Ls, Rs)
        long maxLsRs = Math.max(lastPersonStall.getLeftEmpty(), lastPersonStall.getRightEmpty());
        long minLsRs = Math.min(lastPersonStall.getLeftEmpty(), lastPersonStall.getRightEmpty());
        return (maxLsRs + " " + minLsRs);
    }

    /**
     * Creates an output file w/ solutions to the test cases in the provided file.
     * @param filePath the input file w/ test cases.
     */
    public void runTestCases(String filePath) {
        try {
            PrintWriter pw = new PrintWriter(new File("solution-" + filePath));
            File input = new File(filePath);
            Scanner scan = new Scanner(input);

            int testCases = Integer.valueOf(scan.nextLine());
            for(int i=1; i <= testCases; i++) {
                int numStalls = scan.nextInt();
                int numPeople = scan.nextInt();
                String maxAndMinForLastPerson = getMaxAndMinForLastPerson(numStalls, numPeople);
                System.out.println("Max(Ls, Rs) and Min(Ls, Rs) for [Stalls: " + numStalls + ", People:" + numPeople + "] => " + maxAndMinForLastPerson);
                pw.write("Case #" + i + ": " + maxAndMinForLastPerson + "\n");
            }

            scan.close();
            pw.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testGetEmptyStalls() {
        List<Boolean> stalls = new ArrayList<Boolean>();
        for(int i=0; i < 100; i++)
            stalls.add(false);

        // Left
        assertEquals(0, getNumEmptyLeftStalls(stalls, 0));
        assertEquals(50, getNumEmptyLeftStalls(stalls, 50));
        assertEquals(99, getNumEmptyLeftStalls(stalls, 99));

        // Right
        assertEquals(99, getNumEmptyRightStalls(stalls, 0));
        assertEquals(49, getNumEmptyRightStalls(stalls, 50));
        assertEquals(0, getNumEmptyRightStalls(stalls, 99));

        try {
            // Invalid - throws exceptions
            assertEquals(0, getNumEmptyLeftStalls(stalls, 100));
            assertEquals(0, getNumEmptyLeftStalls(stalls, -1));
        } catch(Exception e) {
            System.out.println("Exception: " + e);
        }
    }

    @Test
    public void testStallInfo() {
        List<BathroomStalls.StallInfo> stallInfos = new ArrayList<BathroomStalls.StallInfo>();
        stallInfos.add(new BathroomStalls.StallInfo(4, 2, 2));
        stallInfos.add(new BathroomStalls.StallInfo(3, 3, 4));
        stallInfos.add(new BathroomStalls.StallInfo(2, 3, 6));
        stallInfos.add(new BathroomStalls.StallInfo(1, 6, 3));

        System.out.println("Before: " + stallInfos);
        assertEquals("[StallInfo{stallIndex=4, leftEmpty=2, rightEmpty=2}, StallInfo{stallIndex=3, leftEmpty=3, " +
                "rightEmpty=4}, StallInfo{stallIndex=2, leftEmpty=3, rightEmpty=6}, StallInfo{stallIndex=1, leftEmpty=6, " +
                "rightEmpty=3}]", stallInfos.toString());

        // Sort by StallInfo's default sorting method
        Collections.sort(stallInfos);

        System.out.println("After: " + stallInfos);
        assertEquals("[StallInfo{stallIndex=1, leftEmpty=6, rightEmpty=3}, StallInfo{stallIndex=2, leftEmpty=3, " +
                "rightEmpty=6}, StallInfo{stallIndex=3, leftEmpty=3, rightEmpty=4}, StallInfo{stallIndex=4, leftEmpty=2, " +
                "rightEmpty=2}]", stallInfos.toString());
    }

    @Test
    public void testMaxAndMinForLastPerson() {
        assertEquals("1 0", getMaxAndMinForLastPerson(4, 2));
        assertEquals("1 0", getMaxAndMinForLastPerson(5, 2));
        assertEquals("1 1", getMaxAndMinForLastPerson(6, 2));
        assertEquals("0 0", getMaxAndMinForLastPerson(1000, 1000));
        assertEquals("500 499", getMaxAndMinForLastPerson(1000, 1));
    }
}
