package codejam;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Comparator;
import java.util.PriorityQueue;
import java.util.Scanner;

import org.junit.Assert;
import org.junit.Test;

public class CBathroomStalls {

	static class PairDiffComparator implements Comparator<Pair> {

		@Override
		public int compare(Pair one, Pair two) {
			long diff1 = one.val2 - one.val1;
			long diff2 = two.val2 - two.val1;
			return Long.compare(diff2, diff1);
		}
		
	}
	
	static class Pair implements Comparable<Pair> {
		long val1;
		long val2;
		long diff;
		
		@Override
		public String toString() {
			return "(" + val1 + ", " + val2 + ")";
		}

		@Override
		public int compareTo(Pair another) {
			long diff1 = val2 - val1;
			long diff2 = another.val2 - another.val1;
			return Long.compare(diff1, diff2);
		}
	}
	
	static class Occupation {
		int closest;
		int farthest;
		int index;
	}

	private Pair solve(long n, long k) {
		return solveDivision(n, k);
	}

	private Occupation occupy(boolean[] stalls) {
		int index = -1;
		int maxClosest = Integer.MIN_VALUE;
		int maxFarthest = Integer.MIN_VALUE;

		for (int i = 1; i < stalls.length - 1; i++) {
			if (!stalls[i]) {
				int empty = 0;
				for (int j = i - 1; j >= 0; j--) {
					if (!stalls[j]) {
						empty++;
					} else {
						break;
					}
				}
				int emptyLeft = empty;

				empty = 0;
				for (int j = i + 1; j < stalls.length; j++) {
					if (!stalls[j]) {
						empty++;
					} else {
						break;
					}
				}

				int emptyRight = empty;

				int closest = Math.min(emptyLeft, emptyRight);
				int farthest = Math.max(emptyLeft, emptyRight);
				if (closest > maxClosest) {
					index = i;
					maxClosest = closest;
					maxFarthest = farthest;
				} else if (closest == maxClosest) {
					if (farthest > maxFarthest) {
						index = i;
						maxClosest = closest;
						maxFarthest = farthest;
					}
				}
			}
		}

		Occupation o = new Occupation();
		o.index = index;
		o.closest = maxClosest;
		o.farthest = maxFarthest;
		return o;
	}

	private Pair solveBrutforce(long n, long k) {
		boolean[] stalls = new boolean[(int) n + 2];
		stalls[0] = true;
		stalls[stalls.length - 1] = true;

		Occupation o = null;
		for (int i = 0; i < k; i++) {
			o = occupy(stalls);
			stalls[o.index] = true;
		}

		Pair pair = new Pair();
		pair.val1 = o.farthest;
		pair.val2 = o.closest;
		return pair;
	}

	@Test
	public void testOccupation() {
		Occupation o = occupy(new boolean[] { true, false, false, false, true });
		Assert.assertEquals(2, o.index);
		Assert.assertEquals(1, o.closest);
		Assert.assertEquals(1, o.farthest);

		o = occupy(new boolean[] { true, false, true });
		Assert.assertEquals(1, o.index);
		Assert.assertEquals(0, o.closest);
		Assert.assertEquals(0, o.farthest);

		o = occupy(new boolean[] { true, false, false, true });
		Assert.assertEquals(1, o.index);
		Assert.assertEquals(0, o.closest);
		Assert.assertEquals(1, o.farthest);

		o = occupy(new boolean[] { true, false, false, false, false, true });
		Assert.assertEquals(2, o.index);
		Assert.assertEquals(1, o.closest);
		Assert.assertEquals(2, o.farthest);

		o = occupy(new boolean[] { true, false, true, false, false, true });
		Assert.assertEquals(3, o.index);
		Assert.assertEquals(0, o.closest);
		Assert.assertEquals(1, o.farthest);
	}

	@Test
	public void testBrutforce() {
		Pair p = solveBrutforce(4, 2);
		Assert.assertEquals(1, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveBrutforce(5, 2);
		Assert.assertEquals(1, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveBrutforce(6, 2);
		Assert.assertEquals(1, p.val1);
		Assert.assertEquals(1, p.val2);

		p = solveBrutforce(1000, 1000);
		Assert.assertEquals(0, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveBrutforce(1000, 1);
		Assert.assertEquals(500, p.val1);
		Assert.assertEquals(499, p.val2);
	}

	@Test
	public void testLarge() {
		 Pair p = solveDivision(500000, 237856);
		 Assert.assertEquals(1, p.val1);
		 Assert.assertEquals(1, p.val2);
	}

	@Test
	public void testCompare() {
		for (int i = 1; i <= 100; i++) {
			for (int j = 1; j <= i; j++) {
				int n = i;
				int k = j;

				Pair p1 = solveBrutforce(n, k);
				Pair p2 = solveBrutforceOptimized(n, k);
				Assert.assertEquals("n = " + n + ", k = " + k, p1.val1, p2.val1);
				Assert.assertEquals(p1.val2, p2.val2);
			}
		}
	}

	@Test
	public void testPerfomance() {
		int n = 10000;
		int k = 1;

		long start1 = System.currentTimeMillis();
		Pair p1 = solveBrutforce(n, k);
		long elapsed1 = System.currentTimeMillis() - start1;

		long start2 = System.currentTimeMillis();
		Pair p2 = solveBrutforceOptimized(n, k);
		long elapsed2 = System.currentTimeMillis() - start2;

		Assert.assertEquals(p1.val1, p2.val1);
		Assert.assertEquals(p1.val2, p2.val2);
		Assert.assertTrue(elapsed2 < elapsed1);
	}
	
	@Test
	public void testPerfomance2() {
		int n = 10000;
		int k = 5000;

		long start1 = System.currentTimeMillis();
		Pair p1 = solveBrutforceOptimized(n, k);
		long elapsed1 = System.currentTimeMillis() - start1;

		long start2 = System.currentTimeMillis();
		Pair p2 = solveDivision(n, k);
		long elapsed2 = System.currentTimeMillis() - start2;

		Assert.assertEquals(p1.val1, p2.val1);
		Assert.assertEquals(p1.val2, p2.val2);
		Assert.assertTrue(elapsed2 < elapsed1);
	}


	int[] left;
	int[] right;

	private Pair solveBrutforceOptimized(long n, long k) {
		boolean[] stalls = new boolean[(int) n + 2];
		stalls[0] = true;
		stalls[stalls.length - 1] = true;

		initLeftRight(stalls);

		Occupation o = null;
		for (int i = 0; i < k; i++) {
			o = occupyOptimized(stalls);

			int dist = 0;
			for (int j = o.index + 1; j < stalls.length; j++) {
				if (!stalls[j]) {
					this.left[j] = dist;
					dist++;
				} else {
					break;
				}
			}

			dist = 0;
			for (int j = o.index - 1; j >= 0; j--) {
				if (!stalls[j]) {
					this.right[j] = dist;
					dist++;
				} else {
					break;
				}
			}

			stalls[o.index] = true;
		}

		Pair pair = new Pair();
		pair.val1 = o.farthest;
		pair.val2 = o.closest;
		return pair;
	}

	private void initLeftRight(boolean[] stalls) {
		left = new int[stalls.length];
		right = new int[stalls.length];

		for (int i = 1; i < stalls.length - 1; i++) {
			left[i] = i - 1;
			right[i] = stalls.length - i - 2;
		}
	}

	private Occupation occupyOptimized(boolean[] stalls) {
		int index = 1;
		int maxClosest = Integer.MIN_VALUE;
		int maxFarthest = Integer.MIN_VALUE;

		for (int i = 1; i < stalls.length - 1; i++) {
			if (!stalls[i]) {
				int emptyLeft = this.left[i];
				int emptyRight = this.right[i];

				int closest = Math.min(emptyLeft, emptyRight);
				int farthest = Math.max(emptyLeft, emptyRight);

				if (closest > maxClosest) {
					index = i;
					maxClosest = closest;
					maxFarthest = farthest;
				} else if (closest == maxClosest) {
					if (farthest > maxFarthest) {
						index = i;
						maxClosest = closest;
						maxFarthest = farthest;
					}
				}
			}
		}

		Occupation o = new Occupation();
		o.index = index;
		o.closest = maxClosest;
		o.farthest = maxFarthest;
		return o;
	}

	@Test
	public void testOccupyOptimized() {
		boolean[] stalls = new boolean[] { true, false, false, false, true };
		initLeftRight(stalls);
		Occupation o = occupyOptimized(stalls);
		Assert.assertEquals(2, o.index);
		Assert.assertEquals(1, o.closest);
		Assert.assertEquals(1, o.farthest);

		stalls = new boolean[] { true, false, true };
		initLeftRight(stalls);
		o = occupyOptimized(stalls);
		Assert.assertEquals(1, o.index);
		Assert.assertEquals(0, o.closest);
		Assert.assertEquals(0, o.farthest);

		stalls = new boolean[] { true, false, false, true };
		initLeftRight(stalls);
		o = occupyOptimized(stalls);
		Assert.assertEquals(1, o.index);
		Assert.assertEquals(0, o.closest);
		Assert.assertEquals(1, o.farthest);

		stalls = new boolean[] { true, false, false, false, false, true };
		initLeftRight(stalls);
		o = occupyOptimized(stalls);
		Assert.assertEquals(2, o.index);
		Assert.assertEquals(1, o.closest);
		Assert.assertEquals(2, o.farthest);
	}

	@Test
	public void testBrutforceOptimized() {
		Pair p = solveBrutforceOptimized(4, 2);
		Assert.assertEquals(1, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveBrutforceOptimized(5, 2);
		Assert.assertEquals(1, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveBrutforceOptimized(6, 2);
		Assert.assertEquals(1, p.val1);
		Assert.assertEquals(1, p.val2);

		p = solveBrutforceOptimized(1000, 1000);
		Assert.assertEquals(0, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveBrutforceOptimized(1000, 1);
		Assert.assertEquals(500, p.val1);
		Assert.assertEquals(499, p.val2);

	}

	int occupied = 0;

	@Test
	public void testHeap() {
		PriorityQueue<Pair> queue = new PriorityQueue<>(10, new PairDiffComparator());
		Pair pair = new Pair();
		pair.val1 = 0;
		pair.val2 = 2;
		queue.add(pair);
		
		pair = new Pair();
		pair.val1 = 5;
		pair.val2 = 10;
		queue.add(pair);
		
		pair = new Pair();
		pair.val1 = 4;
		pair.val2 = 7;
		queue.add(pair);
		
		pair = queue.remove();
		Assert.assertEquals(5, pair.val1);
		Assert.assertEquals(10, pair.val2);
		
		pair = queue.remove();
		Assert.assertEquals(4, pair.val1);
		Assert.assertEquals(7, pair.val2);
	}
	
	private Pair solveDivisionBFS(long start, long end, long k) {
		PriorityQueue<Pair> queue = new PriorityQueue<>(10, new PairDiffComparator());
		Pair pair = new Pair();
		pair.val1 = start;
		pair.val2 = end;
		queue.add(pair);

		long step = 0;

		while (!queue.isEmpty()) {
			step++;

			pair = queue.remove();
			start = pair.val1;
			end = pair.val2;

			long length = end - start;
			if (length < 2) {
				continue;
			}

			long middle = start + (length >> 1);
			if (step >= k) {
				long left = middle - start - 1;
				long right = end - middle - 1;
				Pair answer = new Pair();
				answer.val1 = Math.max(left, right);
				answer.val2 = Math.min(left, right);
				return answer;
			}

			Pair left = new Pair();
			left.val1 = start;
			left.val2 = middle;
			Pair right = new Pair();
			right.val1 = middle;
			right.val2 = end;

			if (end - middle > middle - start) {
				queue.add(right);
				queue.add(left);
			} else {
				queue.add(left);
				queue.add(right);
			}
		}

		return null;
	}

	private Pair solveDivision(long n, long k) {
		occupied = 0;
		Pair pair = solveDivisionBFS(0, n + 1, k);
		return pair;
	}

	@Test
	public void testDivisionMore() {
		Pair p = solveDivision(5, 3);
		Assert.assertEquals(1, p.val1);
		Assert.assertEquals(0, p.val2);
	}
	
	@Test
	public void testDivisionMore2() {
		Pair p = solveDivision(9, 5);
		Assert.assertEquals(1, p.val1);
		Assert.assertEquals(0, p.val2);
	}

	@Test
	public void testDivisionOneItem() {
		Pair p = solveDivision(3, 1);
		Assert.assertEquals(1, p.val1);
		Assert.assertEquals(1, p.val2);

		p = solveDivision(4, 1);
		Assert.assertEquals(2, p.val1);
		Assert.assertEquals(1, p.val2);

		p = solveDivision(1000, 1);
		Assert.assertEquals(500, p.val1);
		Assert.assertEquals(499, p.val2);

		p = solveDivision(1000000, 1);
		Assert.assertEquals(500000, p.val1);
		Assert.assertEquals(499999, p.val2);
	}

	@Test
	public void testDivisionSame() {
		Pair p = solveDivision(3, 3);
		Assert.assertEquals(0, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveDivision(2, 2);
		Assert.assertEquals(0, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveDivision(1, 1);
		Assert.assertEquals(0, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveDivision(1000, 1000);
		Assert.assertEquals(0, p.val1);
		Assert.assertEquals(0, p.val2);
	}

	@Test
	public void testDivisionSmallInput() {
		Pair p = solveDivision(3, 2);
		Assert.assertEquals(0, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveDivision(4, 2);
		Assert.assertEquals(1, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveDivision(5, 2);
		Assert.assertEquals(1, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveDivision(6, 2);
		Assert.assertEquals(1, p.val1);
		Assert.assertEquals(1, p.val2);

		p = solveDivision(1000, 1000);
		Assert.assertEquals(0, p.val1);
		Assert.assertEquals(0, p.val2);

		p = solveDivision(1000, 1);
		Assert.assertEquals(500, p.val1);
		Assert.assertEquals(499, p.val2);
	}

	@Test
	public void testCompareBrutforceOptimizedAndDivision() {
		for (int i = 1; i <= 100; i++) {
			for (int j = 1; j <= i; j++) {
				int n = i;
				int k = j;

				Pair p1 = solveBrutforceOptimized(n, k);
				Pair p2 = solveDivision(n, k);
				Assert.assertEquals("n = " + n + ", k = " + k, p1.val1, p2.val1);
				Assert.assertEquals("n = " + n + ", k = " + k, p1.val2, p2.val2);
			}
		}
	}
	
	@Test
	public void testCompareBrutforceAndDivision() {
		for (int i = 1; i <= 100; i++) {
			for (int j = 1; j <= i; j++) {
				int n = i;
				int k = j;

				Pair p1 = solveBrutforce(n, k);
				Pair p2 = solveDivision(n, k);
				Assert.assertEquals("n = " + n + ", k = " + k, p1.val1, p2.val1);
				Assert.assertEquals("n = " + n + ", k = " + k, p1.val2, p2.val2);
			}
		}
	}

	public static void main(String[] args) throws Exception {
		FileInputStream fis = new FileInputStream("d:\\1\\C-small-2-attempt0.in");
		BufferedReader br = new BufferedReader(new InputStreamReader(fis));
		Scanner scan = new Scanner(br);
		int numberOfCases = scan.nextInt();

		for (int i = 0; i < numberOfCases; i++) {
			long nStalls = scan.nextLong();
			long kPeople = scan.nextLong();

			Pair result = new CBathroomStalls().solve(nStalls, kPeople);
			System.out.println("Case #" + (i + 1) + ": " + result.val1 + " " + result.val2);
		}

		scan.close();
	}

}