import java.io.*;
import java.math.*;
import java.util.ArrayList;
import java.lang.*;

import com.sun.xml.internal.fastinfoset.util.CharArray;

public class Numbers
{
	
	/** Taken from: http://blogger.xs4all.nl/novaloka/archive/2007/09/15/295396.aspx
	   * Returns the correctly rounded square root of a positive BigDecimal. 

	   * The algorithm for taking the square root of a BigDecimal is most 
	   * critical for the speed of your application. This method performs the fast 
	   * Square Root by Coupled Newton Iteration algorithm by Timm Ahrendt,
	   * from the book "Pi, unleashed" by Jrg Arndt in a neat loop.
	   *
	   * @param squarD   number to get the root from (called "d" in the book)
	   * @param rootMC   precision and rounding mode (for the last root "x")
	   *
	   * @return the root of the argument number
	   *
	   * @throws ArithmeticException       if the argument number is negative
	   * @throws IllegalArgumentException  if rootMC has precision 0
	   */

	  public static BigDecimal bigSqrt(BigDecimal squarD, MathContext rootMC)
	  {
	    // Static constants - perhaps initialize in class Vladimir!
	    BigDecimal TWO = new BigDecimal(2);
	    double SQRT_10 = 3.162277660168379332;
	    

	    // General number and precision checking
	    int sign = squarD.signum();
	    if(sign == -1)
	      throw new ArithmeticException("\nSquare root of a negative number: " + squarD);
	    else if(sign == 0)
	      return squarD.round(rootMC);
	    
	    int prec = rootMC.getPrecision();           // the requested precision
	    if(prec == 0)
	      throw new IllegalArgumentException("\nMost roots won't have infinite precision = 0");
	    
	    // Initial precision is that of double numbers 2^63/2 ~ 4E18
	    int BITS = 62;                              // 63-1 an even number of number bits
	    int nInit = 16;                             // precision seems 16 to 18 digits
	    MathContext nMC = new MathContext(18, RoundingMode.HALF_DOWN);
	    
	    
	    // Iteration variables, for the square root x and the reciprocal v
	    BigDecimal x = null, e = null;              // initial x:  x0 ~ sqrt()
	    BigDecimal v = null, g = null;              // initial v:  v0 = 1/(2*x)
	    
	    // Estimate the square root with the foremost 62 bits of squarD
	    BigInteger bi = squarD.unscaledValue();     // bi and scale are a tandem
	    int biLen = bi.bitLength();
	    int shift = Math.max(0, biLen - BITS + (biLen%2 == 0 ? 0 : 1));   // even shift..
	    bi = bi.shiftRight(shift);                  // ..floors to 62 or 63 bit BigInteger
	    
	    double root = Math.sqrt(bi.doubleValue());
	    BigDecimal halfBack = new BigDecimal(BigInteger.ONE.shiftLeft(shift/2));
	    
	    int scale = squarD.scale();
	    if(scale % 2 == 1)                          // add half scales of the root to odds..
	      root *= SQRT_10;                          // 5 -> 2, -5 -> -3 need half a scale more..
	    scale = (int)Math.floor(scale/2.);          // ..where 100 -> 10 shifts the scale
	    
	    // Initial x - use double root - multiply by halfBack to unshift - set new scale
	    x = new BigDecimal(root, nMC);
	    x = x.multiply(halfBack, nMC);                          // x0 ~ sqrt()
	    if(scale != 0)
	      x = x.movePointLeft(scale);

	    if(prec < nInit)                 // for prec 15 root x0 must surely be OK
	      return x.round(rootMC);        // return small prec roots without iterations

	    // Initial v - the reciprocal
	    v = BigDecimal.ONE.divide(TWO.multiply(x), nMC);        // v0 = 1/(2*x)
	    
	    
	    // Collect iteration precisions beforehand
	    ArrayList nPrecs = new ArrayList();
	    
	    assert nInit > 3 : "Never ending loop!";                // assume nInit = 16 <= prec
	    
	    // Let m be the exact digits precision in an earlier! loop
	    for(int m = prec+1; m > nInit; m = m/2 + (m > 100 ? 1 : 2))                      
	      nPrecs.add(new Integer(m));
	    

	    // The loop of "Square Root by Coupled Newton Iteration" for simpletons
	    for(int i = nPrecs.size()-1; i > -1; i--)
	    {
	      // Increase precision - next iteration supplies n exact digits
	      nMC = new MathContext((Integer)nPrecs.get(i), (i%2 == 1) ? RoundingMode.HALF_UP : 
	                                                        RoundingMode.HALF_DOWN);
	      
	      // Next x                                                 // e = d - x^2
	      e = squarD.subtract(x.multiply(x, nMC), nMC);
	      if(i != 0)
	        x = x.add(e.multiply(v, nMC));                          // x += e*v     ~ sqrt()
	      else
	      {
	        x = x.add(e.multiply(v, rootMC), rootMC);               // root x is ready!
	        break;
	      }

	      // Next v                                                 // g = 1 - 2*x*v
	      g = BigDecimal.ONE.subtract(TWO.multiply(x).multiply(v, nMC));
	      
	      v = v.add(g.multiply(v, nMC));                            // v += g*v     ~ 1/2/sqrt()
	    }

	    return x;                        // return sqrt(squarD) with precision of rootMC
	  }
	
	
	public static void main(String args[]) throws Exception
	{
		
		BufferedReader input = new BufferedReader(new InputStreamReader(System.in));

		int N = Integer.parseInt(input.readLine());
		BigDecimal val = new BigDecimal(3);
		BigDecimal val2 = new BigDecimal(5);		
		MathContext mc = MathContext.DECIMAL128;
		
		BigDecimal val3 = bigSqrt(val2, mc);
		val3 = val3.add(val);
		
		for(int tc = 0 ; tc < N ; tc++)
		{
			BigDecimal bd = val3;
			
			String inputText = input.readLine();
			int n = Integer.parseInt(inputText);
			//System.out.println("n = " +  n);
			//System.out.println("val = " + val);
			bd = bd.pow(n);
			String tmp = bd.toString();
			//System.out.println(tmp);
			String output = "";
			int i = tmp.indexOf('.');
			if (i == -1)
			{
				//case where no decimal
				for(int j = 1 ; j <= 3 ; j++)
				{
					if (tmp.length() >= j)
						output = tmp.charAt(tmp.length()-j) + output;
					else
						output = "0" + output;
				}
			}
			else
			{
				int c = 0;
				for(int j = i-1 ; j >= 0 ; j--)
				{
					c++;
					output = tmp.charAt(j) + output;
					if (c==3)
						break;
				}
							
				while (output.length() < 3)
					output = "0" + output;
			}
			
			
			
			System.out.println("Case #" + (tc+1) + ": " + output);
			
		}
		
		
		
	}
	
}