package utils;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.lang.reflect.Array;
import java.lang.reflect.Constructor;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.IdentityHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Bunch of undocumented stuff to make it easier to quickly write code to handle input and output
 * used by some programming puzzles.
 * 
 * @author Wouter Coekaerts <wouter@coekaerts.be>
 */
public class ProblemSolver {
	public class Readable {
		public Readable() {
			outers.add(this);
		}
		
		public int getThisReadingIndex() {
			return getReadingIndex(this.getClass());
		}
	}
	
	public final BufferedReader reader;
	public final PrintStream out;
	public int nextTokenIndex;
	public String[] currentTokens;
	public final DecimalFormat decimalFormat = new DecimalFormat("#.##########");
	private int readCount = 0;
	
	protected Set<Object> outers = Collections.newSetFromMap(new IdentityHashMap<Object, Boolean>());
	{ outers.add(this); }
	
	private Map<Class<?>, Integer> indices = new HashMap<Class<?>, Integer>();
	
	public ProblemSolver(String filename) {
		try {
			if (! filename.startsWith("/") && ! filename.startsWith("."))
				filename = "/tmp/" + filename;
			reader = new BufferedReader(new InputStreamReader(new FileInputStream(filename + ".in")));
			String outfilename = filename + ".out";
			System.out.println("Writing to " + outfilename);
			out = new PrintStream(new FileOutputStream(outfilename));
		} catch (FileNotFoundException e) {
			throw new RuntimeException(e);
		}
	}

	public <T> List<T> readCountAndList(Class<T> clazz) {
		int count = readInt();
		return readList(count, clazz);
	}

	public <T> List<T> readList(int count, Class<T> clazz) {
		List<T> list = new ArrayList<T>(count);
		for (int i = 0; i < count; i++) {
			indices.put(clazz, i);
			T o = read(clazz);
			list.add(o);
		}
		return list;
	}

	public <T> List<T> readListLine(Class<T> clazz) {
		List<T> list = new ArrayList<T>();
		int index = 0;
		do {
			indices.put(clazz, index++);
			T o = read(clazz);
			list.add(o);
		} while (! atNewLine());
		return list;
	}

	public int getReadingIndex(Class<?> clazz) {
		return indices.get(clazz);
	}

	public <T> T read(Class<T> clazz) {
		if (clazz == String.class) {
			return clazz.cast(readToken());
		}
		
		if (clazz.isArray()) {
			return clazz.cast(readArrayLine(clazz.getComponentType()));
		}
		
		try {
			try {
				Constructor<T> stringConstructor = clazz.getConstructor(String.class);
				return stringConstructor.newInstance(readToken());
			} catch (SecurityException e) {
			} catch (NoSuchMethodException e) {
			}
			
			int beforeReadCount = readCount;
			
			final T result;
			Class<?> outerClass = clazz.getDeclaringClass();
			if (outerClass != null) {
				Object outerObject = null;
				for (Object creating : outers) {
					if (outerClass.isAssignableFrom(creating.getClass())) {
						outerObject = creating;
					}
				}
				if (outerObject == null) {
					throw new RuntimeException("Don't know how to get outer class of " + clazz);
				}
				result = clazz.getConstructor(outerClass).newInstance(outerObject);
			} else {
				result = clazz.newInstance();
			}
			outers.remove(result);
			if (beforeReadCount == readCount) {
				throw new RuntimeException("Trying to read " + clazz + " using no-args constructor didn't read anything");
			}
			return result;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	protected boolean atNewLine() {
		return currentTokens == null || nextTokenIndex == currentTokens.length;
	}

	public String readToken() {
		if (atNewLine()) {
			currentTokens = readLine().split(" ");
			nextTokenIndex = 0;
		} else {
			readCount++;
		}
	
		return currentTokens[nextTokenIndex++];
	}

	public String readLine() {
		readCount++;
		try {
			String line = reader.readLine();
			return line;
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public int readInt() {
		return Integer.parseInt(readToken());
	}

	@SuppressWarnings("unchecked")
	private <T> T[] toArray(Collection<T> collection, Class<T> componentType) {
		return collection.toArray((T[])Array.newInstance(componentType, collection.size()));
	}

	public <T> T[] readArray(int count, Class<T> componentType) {
		return this.<T>toArray(readList(count, componentType), componentType);
	}

	public <T> T[] readArrayLine(Class<T> componentType) {
		return this.<T>toArray(readListLine(componentType), componentType);
	}

	public String[] readStringArray() {
		return readLine().split(" ");
	}

	public int[] readIntArray(int expectedLength) {
		int[] result = readIntArray();
		if (result.length != expectedLength) {
			throw new RuntimeException("expected " + expectedLength + " numbers, got " + result.length);
		}
		return result;
	}

	public int[] readIntArray() {
		String[] strings = readStringArray();
		int[] result = new int[strings.length];
		for (int i=0; i < strings.length; i++) {
			result[i] = Integer.parseInt(strings[i]);
		}
		return result;
	}

	public int[][] readIntMatrix(int height, int width) {
		int[][] matrix = new int[height][width];
		for (int y = 0; y < height; y++) {
			String[] cells = readStringArray();
			assert width == -1 || width == cells.length;
			for (int x = 0; x < cells.length; x++) {
				matrix[y][x] = Integer.parseInt(cells[x]);
			}
		}
		return matrix;
	}
	
	public double[] readDoubleArray() {
		String[] strings = readStringArray();
		double[] result = new double[strings.length];
		for (int i=0; i < strings.length; i++) {
			result[i] = Double.parseDouble(strings[i]);
		}
		return result;
	}

	public String format(Object o) {
		if (o instanceof String) {
			return (String)o;
		} else if (o instanceof Iterable<?>) {
			return formatCollection((Iterable<?>)o, " ");
		} else if (o instanceof Object[]){
			return formatArray((Object[])o, " ");
		} else if (o instanceof Double) {
			return formatDouble(((Double)o).doubleValue());
		} else {
			return o.toString();
		}
	}

	public String formatDouble(double d) {
		String s = decimalFormat.format(d);
		return "-0".equals(s) ? "0" : s;
	}

	public String formatArray(Object[] array, String separator) {
		StringBuilder builder = new StringBuilder();
		for (Object o : array) {
			builder.append(format(o)).append(separator);
		}
		return builder.substring(0, builder.length()-1);
	}

	public String formatCollection(Iterable<?> iter, String separator) {
		StringBuilder builder = new StringBuilder();
		for (Object o : iter) {
			builder.append(format(o)).append(separator);
		}
		return builder.substring(0, builder.length()-1);
	}

	public String formatMatrix(Object[][] o) {
		return formatArray(o, "\n");
	}

	public void print(String line) {
		System.out.println(line);
		out.println(line);
	}

	public void print(Object o) {
		print(format(o));
	}
	
	/**
	 * Convenience superclass that implements equals and hashcode based on a given list of properties.
	 * The constructor should call {@link #initKey(Object...)}
	 */
	public class Keyed extends Readable {
		private Key key;

		public Keyed() {
		}
		
		protected void initKey(Object... keys) {
			key = new Key(keys);
		}
		
		@Override
		public boolean equals(Object obj) {
			if (! this.getClass().isInstance(obj))
				return false;
			return key.equals(((Keyed)obj).key);
		}
		
		@Override
		public int hashCode() {
			return key.hashCode();
		}
		
		@Override
		public String toString() {
			return this.getClass().getSimpleName() + '[' + key + ']';
		}
	}
	
	/**
	 * Returns a composite key containing the given values; to be used in Maps when you're too lazy to write
	 * a specific class for this case.
	 */
	public Key key(Object ... values) {
		return new Key(values);
	}
	
	/**
	 * Composite key to use in Maps.
	 */
	public class Key {
		public final Object[] values;
		
		public Key(Object[] values) {
			this.values = values;
		}
		
		@Override
		public String toString() {
			return format(values);
		}

		@Override
		public int hashCode() {
			return Arrays.deepHashCode(values);
		}

		@Override
		public boolean equals(Object obj) {
			// technically we should allow this, but if this ever happens in our puzzles
			// its probably a mistake
			if (! (obj instanceof Key))
				throw new RuntimeException("shouldn't be comparing a key to " + obj);
			Key other = (Key) obj;
			if (values.length != other.values.length) {
				// same thing here
				throw new RuntimeException("different number of values, shouldn't be comparing these: " + this + " and " + obj);
			}
			return Arrays.deepEquals(values, other.values);
		}
	}

}
