package utils;

import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.PriorityQueue;

/**
 * A only-half-finished graph path searching thingie.
 * 
 * @author Wouter Coekaerts <wouter@coekaerts.be>
 *
 * @param <N> Type of nodes
 */
public abstract class GraphSearcher<N> {
	private Comparator<ScoredPath> SCORE_COMPARATOR = new Comparator<ScoredPath>() {
		@Override
		public int compare(ScoredPath o1, ScoredPath o2) {
			return o1.h <= o2.h ? -1 : o1.h == o2.h ? 0 : 1;
		}
	};
	
	public class ScoredPath {
		public final N node;
		public final ScoredPath previous;
		public final int score;
		public final int minScoreToDestination;
		public final int h;
		
		public ScoredPath(N node, ScoredPath previous, int score) {
			super();
			this.node = node;
			this.previous = previous;
			this.score = score;
			this.minScoreToDestination = minScoreToDestination(node);
			this.h = score + minScoreToDestination;
		}
		
		@Override
		public String toString() {
			String prefix = (previous == null) ? "" : (previous + "->"); 
			return prefix + node.toString() + " (s=" + score + ")";
		}
	}
	
	abstract public Collection<ScoredPath> getNext(ScoredPath path);
	abstract public boolean isDestination(N node);
	
	public Comparator<ScoredPath> getComparator() {
		return SCORE_COMPARATOR;
	}
	
	public int minScoreToDestination(N node) {
		return 0;
	}
	
	public ScoredPath findBestPath(ScoredPath start) {
		HashMap<N, ScoredPath> bestPaths = new HashMap<N, ScoredPath>();
		PriorityQueue<ScoredPath> nodesToVisit = new PriorityQueue<ScoredPath>(11, getComparator());
		ScoredPath bestPathToDestination = null;
		
		nodesToVisit.add(start);
		
		while (!nodesToVisit.isEmpty()) {
			ScoredPath current = nodesToVisit.remove();
			ScoredPath previousBestToCurrent = bestPaths.get(current.node);
			if (previousBestToCurrent == null || previousBestToCurrent == current) { // if this is still the best path
				for (ScoredPath next : getNext(current)) {
					ScoredPath previousBestToNext = bestPaths.get(next.node);
					if (previousBestToNext == null || getComparator().compare(previousBestToNext, next) > 0) { // found a better path
//						debug("New best path: " + next);
						bestPaths.put(next.node, next);
						nodesToVisit.add(next);
						if (isDestination(next.node)) {
							if (bestPathToDestination == null || getComparator().compare(bestPathToDestination, next) > 0)
//							debug("(destination)");
							bestPathToDestination = next;
						}
					}
				}
			}
		}
		
		debug("Best: " + bestPathToDestination);
		
		return bestPathToDestination;		
	}
	
	void debug(String s) {
		System.out.println(s);
	}
}
