package hu.herba.codejam;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.Scanner;

/**
 * The Dutch computer scientist Edsger Dijkstra made many important contributions to the field, including the shortest path finding algorithm that bears his
 * name. This problem is not about that algorithm.
 * 
 * You were marked down one point on an algorithms exam for misspelling "Dijkstra" -- between D and stra, you wrote some number of characters, each of which was
 * either i, j, or k. You are prepared to argue to get your point back using quaternions, an actual number system (extended from complex numbers) with the
 * following multiplicative structure:
 * 
 * To multiply one quaternion by another, look at the row for the first quaternion and the column for the second quaternion. For example, to multiply i by j,
 * look in the row for i and the column for j to find that the answer is k. To multiply j by i, look in the row for j and the column for i to find that the
 * answer is -k.
 * 
 * As you can see from the above examples, the quaternions are not commutative -- that is, there are some a and b for which a * b != b * a. However they are
 * associative -- for any a, b, and c, it's true that a * (b * c) = (a * b) * c.
 * 
 * Negative signs before quaternions work as they normally do -- for any quaternions a and b, it's true that -a * -b = a * b, and -a * b = a * -b = -(a * b).
 * 
 * You want to argue that your misspelling was equivalent to the correct spelling ijk by showing that you can split your string of is, js, and ks in two places,
 * forming three substrings, such that the leftmost substring reduces (under quaternion multiplication) to i, the middle substring reduces to j, and the right
 * substring reduces to k. (For example, jij would be interpreted as j * i * j; j * i is -k, and -k * j is i, so jij reduces to i.) If this is possible, you
 * will get your point back. Can you find a way to do it?
 * 
 * Input
 * 
 * The first line of the input gives the number of test cases, T. T test cases follow. Each consists of one line with two space-separated integers L and X,
 * followed by another line with L characters, all of which are i, j, or k. Note that the string never contains negative signs, 1s, or any other characters. The
 * string that you are to evaluate is the given string of L characters repeated X times. For instance, for L = 4, X = 3, and the given string kiij, your input
 * string would be kiijkiijkiij.
 * 
 * Output
 * 
 * For each test case, output one line containing "Case #x: y", where x is the test case number (starting from 1) and y is either YES or NO, depending on
 * whether the string can be broken into three parts that reduce to i, j, and k, in that order, as described above.
 * 
 * @author Zoltn
 * 
 */
public class Dijkstra {
	// private final Value value1 = new Value('1');
	private final Value valueI = new FinalValue('i');
	private final Value valueJ = new FinalValue('j');
	private final Value valueK = new FinalValue('k');

	public class FinalValue extends Value {
		public FinalValue() {
			super();
		}

		public FinalValue(char ch) {
			super(ch);
		}

		@Override
		protected void updateTo(char newValue, boolean newPositive) {
			throw new IllegalStateException("This value (" + this + ") is not modifiable!");
		}
	}

	public class Value {
		private char c; // 1,i,j,k
		private boolean positive;

		public Value() {
			this('1');
		}

		public Value(char ch) {
			this.c = ch;
			positive = true;
		}

		public Value update(char nextChar) {
			switch (nextChar) {
			case 'i':
				applyI();
				break;
			case 'j':
				applyJ();
				break;
			case 'k':
				applyK();
				break;
			default:
				throw new IllegalArgumentException("Invalid character: " + nextChar);
			}
			return this;
		}

		private void applyI() {
			switch (c) {
			case 'i':
				updateTo('1', !positive);
				break;
			case 'j':
				updateTo('k', !positive);
				break;
			case 'k':
				updateTo('j', positive);
				break;
			case '1':
			default:
				updateTo('i', positive);
			}
		}

		private void applyJ() {
			switch (c) {
			case 'i':
				updateTo('k', positive);
				break;
			case 'j':
				updateTo('1', !positive);
				break;
			case 'k':
				updateTo('i', !positive);
				break;
			case '1':
			default:
				updateTo('j', positive);
			}
		}

		private void applyK() {
			switch (c) {
			case 'i':
				updateTo('j', !positive);
				break;
			case 'j':
				updateTo('i', positive);
				break;
			case 'k':
				updateTo('1', !positive);
				break;
			case '1':
			default:
				updateTo('k', positive);
			}
		}

		protected void updateTo(char newValue, boolean newPositive) {
			c = newValue;
			this.positive = newPositive;
		}

		@Override
		public String toString() {
			return (positive ? "" : "-") + c;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = (prime * result) + getOuterType().hashCode();
			result = (prime * result) + c;
			result = (prime * result) + (positive ? 1231 : 1237);
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj) {
				return true;
			}
			if (obj == null) {
				return false;
			}
			if (!(obj instanceof Value)) {
				return false;
			}
			Value other = (Value) obj;
			if (!getOuterType().equals(other.getOuterType())) {
				return false;
			}
			if (c != other.c) {
				return false;
			}
			if (positive != other.positive) {
				return false;
			}
			return true;
		}

		private Dijkstra getOuterType() {
			return Dijkstra.this;
		}
	}

	public static void main(String[] args) {
		if (args.length < 1) {
			// process all files in input folder
			for (File input : new File("./input").listFiles()) {
				if (input.getName().endsWith(".in")) {
					System.out.println("Processing: " + input.getName() + "...");
					new Dijkstra().process(input);
				}
			}
		} else {
			File input = new File("./input", args[0]);
			if (!input.canRead()) {
				error("Input file not found: " + input.getAbsolutePath());
			}
			new Dijkstra().process(input);
		}
		System.out.println("READY");
	}

	private static void error(String msg) {
		System.err.println(msg);
		System.exit(-1);

	}

	private void process(File input) {
		// process input file
		try {
			Scanner scan = new Scanner(input);
			// open output
			PrintWriter out = new PrintWriter(new File("./output", input.getName().replace(".in", ".out")));
			try {
				// number of test cases
				int T = scan.nextInt();
				for (int i = 0; i < T; i++) {
					processTestCase(i + 1, scan, out);
				}
			} finally {
				out.close();
			}
		} catch (FileNotFoundException e) {
			error(e.getMessage());
		}

	}

	private void processTestCase(int caseNum, Scanner scan, PrintWriter out) {
		boolean debug = (caseNum == 0);
		// get X, R, C
		StringBuilder result = new StringBuilder("Case #" + caseNum + ": ");
		int L = scan.nextInt(); // length of pattern
		int X = scan.nextInt(); // repeat pattern
		String S = scan.next();
		if (debug) {
			System.out.print(caseNum + "# " + L + ", " + X + ", " + (L > 10 ? S.substring(0, 10) + "..." : S) + " -> ");
		}
		// boolean valid = checkProduct(L, X, S);
		// whole product should be i*j*k = -1
		// if (valid) {
		boolean valid = false;
		Value curVal = new Value('1');
		Value[] expectedValues = new Value[] { valueI, valueJ, valueK };
		int pos = 0;
		Value expectedValue = expectedValues[0];
		if (debug) {
			System.out.print(curVal);
		}

		for (int i = 0; i < X; i++) {
			for (int j = 0; j < L; j++) {
				curVal.update(S.charAt(j));
				if (debug) {
					System.out.print("(" + S.charAt(j) + ")" + curVal);
				}
				if (curVal.equals(expectedValue)) {
					if (debug) {
						System.out.println("!");
					}
					// go to next value
					if (pos++ < 2) {
						expectedValue = expectedValues[pos];
					} else { // the rest must be 1
						expectedValue = new Value('1');
					}
					curVal = new Value();
					if (debug) {
						System.out.print(curVal);
					}
				}
			}
		}
		if (curVal.equals(expectedValue)) {
			if (debug) {
				System.out.println(" OK");
			}
			valid = true;
		}
		// }
		result.append(valid ? "YES" : "NO");
		if (debug) {
			System.out.println("\nResult: " + valid);
		} else if (!valid) {
			System.out.println(caseNum + "# " + L + ", " + X + ", " + (L > 10 ? S.substring(0, 10) + "..." : S) + " -> " + valid);
		}
		out.println(result.toString());
	}

	private boolean checkProduct(int L, int X, String S) {
		Value curVal = new Value('1');
		for (int i = 0; i < X; i++) {
			for (int j = 0; j < L; j++) {
				curVal.update(S.charAt(j));
			}
		}
		// System.out.println(L + ", " + X + ", " + (L > 10 ? S.substring(0, 10) + "..." : S) + " PROD = " + curVal);
		// result must be -1
		return (curVal.c == '1') && !curVal.positive;
	}
}
