import java.io.File;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Scanner;


public final class GooglereseTranslator {

	private final Map<Character, Character> dictionary = new Hashtable<Character, Character>(26);
	{
		dictionary.put('q', 'z');  //hint :)
	}
	
	public GooglereseTranslator (final Map<String, String> translationExamples) {
		for (final Map.Entry<String, String> exampleLine : translationExamples.entrySet()) {
			final String lineGooglerese = exampleLine.getKey();
			final String lineEnglish = exampleLine.getValue();

			for (int i = 0; i < lineGooglerese.length() && i < lineEnglish.length(); ++i) {
				char googlereseChar = lineGooglerese.charAt(i);
				char englishChar = lineEnglish.charAt(i);

				final Character expectedEnglishChar = dictionary.get(googlereseChar);
				if (expectedEnglishChar != null) {
					//check validity
					if (expectedEnglishChar.charValue() != englishChar)
						throw new RuntimeException ("invalid ; already had '" + googlereseChar + "'->'" + expectedEnglishChar + "' ; found newly '" + googlereseChar + "'->'" + englishChar + "'");
					else
						continue;  //already registered
				} else {
					dictionary.put(googlereseChar, englishChar);
				}
			}
		}

		mapLastEntry();
	}

	private void mapLastEntry () {
		final char firstCheckChar = 'a';
		final char finalCheckChar = 'z';

		for (char curCheckChar = firstCheckChar; curCheckChar <= finalCheckChar; ++curCheckChar) {
			if (!dictionary.containsKey(curCheckChar)) {
				for (char curEnglishChar = firstCheckChar; curEnglishChar <= finalCheckChar; ++curEnglishChar)
					if (!dictionary.containsValue(curEnglishChar)) {
						dictionary.put(curCheckChar, curEnglishChar);
						return;  //found it
					}
			}
		}
	}

	public String translate (final String googlereseText) {
		final StringBuilder englishText = new StringBuilder();
		for (int i = 0; i < googlereseText.length(); ++i) {
			englishText.append(dictionary.get(googlereseText.charAt(i)));
		}
		return englishText.toString();
	}
	

	private static final String OUTPUT_FILE_EXTENSION = "out";
	private static final String OUTPUT_LINE_FORMAT = "Case #%d: %s";

	public static void main (final String[] args) throws Exception {
		//input init
		if (args.length < 1)
			throw new IllegalArgumentException();

		final String inputFileName = args[0];
		final int lastDotIndex = inputFileName.lastIndexOf('.');
		final String fileName = (lastDotIndex < 0) ? inputFileName : inputFileName.substring(0, lastDotIndex);
		final String outputFileName = fileName + "." + OUTPUT_FILE_EXTENSION;

		//translator init
		final Map<String, String> translationExamples = new HashMap<String, String>(3);
		translationExamples.put(
				"ejp mysljylc kd kxveddknmc re jsicpdrysi",
				"our language is impossible to understand");
		translationExamples.put(
				"rbcpc ypc rtcsra dkh wyfrepkym veddknkmkrkcd",
				"there are twenty six factorial possibilities");
		translationExamples.put(
				"de kr kd eoya kw aej tysr re ujdr lkgc jv",
				"so it is okay if you want to just give up");

		final GooglereseTranslator translator = new GooglereseTranslator(translationExamples);

		System.out.println(translator.dictionary);

		//translate
		final File inputFile = new File(inputFileName);
		final File outputFile = new File(outputFileName);
		Scanner inputScanner = null;
		PrintWriter outputWriter = null;
		try {
			inputScanner = new Scanner(inputFile);
			outputWriter = new PrintWriter(outputFile);
			final int numberLines = inputScanner.nextInt();
			inputScanner.nextLine();  //go to first text line
			for (int i = 0; i < numberLines; ++i) {
				final String googlereseText = inputScanner.nextLine();
				final String englishTranslation = translator.translate(googlereseText);
				outputWriter.println(String.format(OUTPUT_LINE_FORMAT, i+1, englishTranslation));
			}
		} catch (final NoSuchElementException noSuchElementException) {
			System.err.println("invalid input");
			throw noSuchElementException;
		} finally {
			if (inputScanner != null) {
				inputScanner.close();
			}
			if (outputWriter != null) {
				outputWriter.flush();
				outputWriter.close();
			}
		}
	}

}
