package gcj;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.math.*;

/**
 * Solution to Google Code Jam Problem.
 * 
 * @author dan
 */
public class DigitsCase {
	
	static final BigDecimal TWO = new BigDecimal(2);               // 2
	  
	  static int extraPrecision = 1;        // number of digits extra precision
	  
	  /**
	   * square root method for BigDecimal numbers, also for small precision numbers
	   * initialization part: sets global variables for sqrtProcedure()
	   * hence the synchronized class lock
	   */
	  synchronized public static BigDecimal square(BigDecimal bd, MathContext mc)
	  { 
	    d = bd;                                   // bd global variable
	    double num = d.doubleValue();             // bd to double
	    
	    if(mc.getPrecision() == 0)
	      throw new IllegalArgumentException("\nRoots need a MathContext precision > 0");
	    if(num < 0.)
	      throw new ArithmeticException("\nCannot calculate the square root of a negative number");
	    if(num == 0.)
	      return bd.round(mc);                    // return sqrt(0) immediately
	    
	    if(mc.getPrecision() < 50)                // small precision is buggy..
	      extraPrecision += 5;                    // ..make more precise
	    int startPrecision = 1;                   // default first precision
	    
	    /* create the initial values for the iteration procedure:
	     * x0:  x ~ sqrt(d)
	     * v0:  v = 1/(2*x)
	     */
	    if(num == Double.POSITIVE_INFINITY)       // d > 1.7E308
	    {
	      BigInteger bi = d.unscaledValue();
	      int biLen = bi.bitLength();
	      int biSqrtLen = biLen/2;                // floors it too
	      
	      bi = bi.shiftRight(biSqrtLen);          // bad guess sqrt(d)
	      x = new BigDecimal(bi);                 // x ~ sqrt(d)
	      
	      MathContext mm = new MathContext(5, RoundingMode.HALF_DOWN);   // minimal precision
	      extraPrecision += 10;                   // make up for it later
	      
	      v = BigDecimal.ONE.divide(TWO.multiply(x, mm), mm);   // v = 1/(2*x) 
	    }
	    else                                      // d < 1.7E10^308  (the usual numbers)
	    {
	      double s = Math.sqrt(num);
	      x = new BigDecimal(s);                  // x = sqrt(d)
	      v = new BigDecimal(1./2./s);            // v = 1/2/x 
	      // works because Double.MIN_VALUE * Double.MAX_VALUE ~ 9E-16, so: v > 0
	      
	      startPrecision = 16;                    // double sqrt approximation accuracy
	    }
	    
	    digits = mc.getPrecision() + extraPrecision;        // global limit for procedure
	    
	    // create initial MathContext(precision, RoundingMode)
	    MathContext n = new MathContext(startPrecision, mc.getRoundingMode());
	    
	    BigDecimal result = sqrtProcedure(n);    // start sqrt iteration!
	    
	    // end procedure destructors:
	    x = null; v = null; 
	    g = null; e = null;

	    return result.round(mc);           // return square root using argument precision
	  }
	  
	  // these global variables require class synchronization on square()
	  private static int digits;          // final precision
	  private static BigDecimal d;        // number to be square rooted
	  private static BigDecimal x, v;     // iteration values
	  private static BigDecimal g, e;     // temp values
	  
	  /**
	   * Square root by coupled Newton iteration, sqrtProcedure() is the iteration part
	   * I adopted the Algorithm from the book "Pi-unleashed", so now it looks more natural
	   * I give sparse math comments from the book, it assumes argument mc precision >= 1
	   **/
	  private static BigDecimal sqrtProcedure(MathContext mc)
	  {
	    // next v                                         // g = 1 - 2*x*v
	    g = BigDecimal.ONE.subtract(TWO.multiply(x, mc).multiply(v, mc), mc);
	    v = v.add(g.multiply(v, mc), mc);                 // v += g*v        ~ 1/2/sqrt(d)
	    
	    // next x
	    e = d.subtract(x.multiply(x, mc), mc);            // e = d - x^2
	    x = x.add(e.multiply(v, mc), mc);                 // x += e*v        ~ sqrt(d)
	    
	    // increase precision
	    int m = mc.getPrecision();
	    if(m < 2)
	      m++;
	    else
	      m = m*2 - 1;                // next Newton iteration supplies so many exact digits
	    
	    if(m < 2*digits)                                    // digits limit not yet reached?
	    {
	      mc = new MathContext(m, mc.getRoundingMode());    // apply new precision
	      sqrtProcedure(mc);                                // next iteration
	    }
	    
	    return x;                                           // returns the iterated square roots
	  }

	  // PP program cubic() root method + general root method

	int _numDigs;
	
	/**
	 * Constructor
	 */
	public DigitsCase ( int numDigs ) {
		_numDigs = numDigs;
		
		//System.out.println("5! is " + Fact(5));
		//System.out.println("3^10 mod 1000 is " + BigExp(3, 10) + ":" + (Math.pow( 3, 10 )) );
		//System.out.println( "TestCase added: " + _vec1 + " vec1, " + _vec2 + " vec2" );
	}

	/**
	 * Entry point
	 * 
	 * @param args unused.
	 */
	public static void main ( String[] args ) throws Exception {
		String fileName = "C-small-attempt1.in";
		File f = new File(fileName);
		BufferedReader br = new BufferedReader( new FileReader( f ) );
		String readLine;
		readLine = br.readLine( );
		int numCases = Integer.parseInt( readLine );
		
		List<DigitsCase> cases = new ArrayList<DigitsCase>( );
		
		for( int i = 0; i < numCases; i++ )
		{
			readLine = br.readLine( );
			
			cases.add( new DigitsCase( Integer.parseInt( readLine ) ) );
		}
		br.close( );
		
		String newLine = "";
		BufferedWriter bw = new BufferedWriter( new FileWriter( new File( "out.txt" ) ) );
		for( int i = 0; i < cases.size( ); i++ )
		{
			bw.write( newLine );
			DigitsCase tc = cases.get( i );
			bw.write( "Case #" + (i+1) + ": " + tc.ProcessTest( ) );
			newLine = "\r\n";
		}
		bw.close( );
	}

	/**
	 * @return
	 */
	public String ProcessTest() {
		String ret = "";
		
		BigDecimal sum = new BigDecimal(1);
		BigDecimal stuff = new BigDecimal(5);
		MathContext m = new MathContext(_numDigs * 2, RoundingMode.DOWN);
		BigDecimal result = square(stuff, m);
		result = result.add( new BigDecimal(3) );
		// inside sq is always 5
		
		for(int i=0; i<_numDigs; i++)
		{
			sum = sum.multiply( result );
		}
		
		// floating point error - schmerror
		//long sum = (int)(BigExp(intPart, _numDigs)*((intPart) + ((double)(sqPart*BigExp(2,_numDigs))*Math.sqrt( 5 )))) + 5*(_numDigs%1000);
		
		/*double fracPart = 0;
		// binomial expanion???
		double sum = 0;
		for(int i=0; i<=_numDigs; i++)
		{
			if( (_numDigs - i)%2 == 0)
			{
				sum += ((double)Fact(_numDigs))/(Fact(i)*Fact(i-_numDigs))*(BigExp(3, i))*BigExp(5, (_numDigs - i) / 2);
			}
			else
			{
				sum += ((double)Fact(_numDigs))/(Fact(i)*Fact(i-_numDigs))*(BigExp(3, i))*Math.pow( Math.sqrt(5), (_numDigs - i) );
			}
			while(sum > 1000)
				sum -= 1000;
		}*/
		
		BigDecimal r = sum.remainder( new BigDecimal(1000 ) );
		long summ = r.longValue( );
		if(summ<0)
			summ = -summ;
		
		ret = new DecimalFormat("000").format( summ );
		return ret;
	}
	
	public int BigExp( int n, int e )
	{
		int c = 1;
		for(int ep = 0; ep <= e; ep++)
		{
			c = (n*c) % 1000;
		}
		return c;
	}
	
	public int Fact( int e )
	{
		int c = 1;
		for(int ep = 1; ep <= e; ep++)
		{
			c = (ep*c);
		}
		return c;
	}
}

/**
 * class Root has static methods for obtaining roots of a BigDecimal
 * Uses "square root by coupled Newton iteration" for huge floating point numbers
 * (this algorithm in Java performs 6% better than normal Newton iteration)
 * square() on a 400Mhz: digits precision => elapsed seconds: 
 *                       5000 => 7s, 10000 => 28s, 20000 => 104s
 * Algorithm 11.11 from Arndt & Haenel, 2000, "Pi-unleashed", page 148 
 */

final class Root
{
  static final BigDecimal TWO = new BigDecimal(2);               // 2
  
  static int extraPrecision = 1;        // number of digits extra precision
  
  /**
   * square root method for BigDecimal numbers, also for small precision numbers
   * initialization part: sets global variables for sqrtProcedure()
   * hence the synchronized class lock
   */
  synchronized public static BigDecimal square(BigDecimal bd, MathContext mc)
  { 
    d = bd;                                   // bd global variable
    double num = d.doubleValue();             // bd to double
    
    if(mc.getPrecision() == 0)
      throw new IllegalArgumentException("\nRoots need a MathContext precision > 0");
    if(num < 0.)
      throw new ArithmeticException("\nCannot calculate the square root of a negative number");
    if(num == 0.)
      return bd.round(mc);                    // return sqrt(0) immediately
    
    if(mc.getPrecision() < 50)                // small precision is buggy..
      extraPrecision += 5;                    // ..make more precise
    int startPrecision = 1;                   // default first precision
    
    /* create the initial values for the iteration procedure:
     * x0:  x ~ sqrt(d)
     * v0:  v = 1/(2*x)
     */
    if(num == Double.POSITIVE_INFINITY)       // d > 1.7E308
    {
      BigInteger bi = d.unscaledValue();
      int biLen = bi.bitLength();
      int biSqrtLen = biLen/2;                // floors it too
      
      bi = bi.shiftRight(biSqrtLen);          // bad guess sqrt(d)
      x = new BigDecimal(bi);                 // x ~ sqrt(d)
      
      MathContext mm = new MathContext(5, RoundingMode.HALF_DOWN);   // minimal precision
      extraPrecision += 10;                   // make up for it later
      
      v = BigDecimal.ONE.divide(TWO.multiply(x, mm), mm);   // v = 1/(2*x) 
    }
    else                                      // d < 1.7E10^308  (the usual numbers)
    {
      double s = Math.sqrt(num);
      x = new BigDecimal(s);                  // x = sqrt(d)
      v = new BigDecimal(1./2./s);            // v = 1/2/x 
      // works because Double.MIN_VALUE * Double.MAX_VALUE ~ 9E-16, so: v > 0
      
      startPrecision = 16;                    // double sqrt approximation accuracy
    }
    
    digits = mc.getPrecision() + extraPrecision;        // global limit for procedure
    
    // create initial MathContext(precision, RoundingMode)
    MathContext n = new MathContext(startPrecision, mc.getRoundingMode());
    
    BigDecimal result = sqrtProcedure(n);    // start sqrt iteration!
    
    // end procedure destructors:
    x = null; v = null; 
    g = null; e = null;

    return result.round(mc);           // return square root using argument precision
  }
  
  // these global variables require class synchronization on square()
  private static int digits;          // final precision
  private static BigDecimal d;        // number to be square rooted
  private static BigDecimal x, v;     // iteration values
  private static BigDecimal g, e;     // temp values
  
  /**
   * Square root by coupled Newton iteration, sqrtProcedure() is the iteration part
   * I adopted the Algorithm from the book "Pi-unleashed", so now it looks more natural
   * I give sparse math comments from the book, it assumes argument mc precision >= 1
   **/
  private static BigDecimal sqrtProcedure(MathContext mc)
  {
    // next v                                         // g = 1 - 2*x*v
    g = BigDecimal.ONE.subtract(TWO.multiply(x, mc).multiply(v, mc), mc);
    v = v.add(g.multiply(v, mc), mc);                 // v += g*v        ~ 1/2/sqrt(d)
    
    // next x
    e = d.subtract(x.multiply(x, mc), mc);            // e = d - x^2
    x = x.add(e.multiply(v, mc), mc);                 // x += e*v        ~ sqrt(d)
    
    // increase precision
    int m = mc.getPrecision();
    if(m < 2)
      m++;
    else
      m = m*2 - 1;                // next Newton iteration supplies so many exact digits
    
    if(m < 2*digits)                                    // digits limit not yet reached?
    {
      mc = new MathContext(m, mc.getRoundingMode());    // apply new precision
      sqrtProcedure(mc);                                // next iteration
    }
    
    return x;                                           // returns the iterated square roots
  }

  // PP program cubic() root method + general root method
  
  
  public static void main(String[] args)         // QQ test method
  {
    BigDecimal bd = null;
    int precision = 60;
    MathContext n = null;
    
    if(args.length >= 2)
    {
      bd = new BigDecimal(args[0]);
      precision = Integer.parseInt(args[1]);
      n = new MathContext(precision, RoundingMode.DOWN);
    }
    else if(args.length <= 1)
    {
      if(args.length == 1)
        precision = Integer.parseInt(args[0]);
      
      String str = "1";
      for(int i = 1; i < 310; i++)
        str += "0";
      str += "1";
      
      // compare MATLAB (MATLAB 5 + MAPLE toolbox) at Pentium 400Mhz
      // digits(2000); tic; vpa sqrt(10^310+1); toc
      // elapsed_time = 110 ms
      // MATLAB bug: last digit of 10000 digit numbers screws up!
      // this Java class is six times slower: 660 ms, but very precise
      
      bd = new BigDecimal(str);
      n = new MathContext(precision, RoundingMode.DOWN);
    }
    
    // Measure time for square root
    long elapsedMillis = System.currentTimeMillis();
    BigDecimal result = square(bd, n);
    elapsedMillis = System.currentTimeMillis() - elapsedMillis;
    
    System.out.println("SQRT=.    .    .    .    .    .    .    .    .    .    .    .\n" + 
                       result.toPlainString() + 
                       "\nelapsed time: " + elapsedMillis + " ms");
      
    // perform test on the result
    BigDecimal rest = result.multiply(result, n).subtract(bd, n);     // sqrt(bd)^2 - bd
    
    int test = Math.abs(rest.unscaledValue().intValue());
    int max = 3;
    
    if(test > max)                // check if it is valid
       System.out.println("rest= " + rest.toPlainString());
    else
       System.out.println("sqrt OK!");          // valid!
  }
}
