package eric;

import java.io.*;
import java.util.*;
import java.text.*;
import java.awt.geom.Point2D;

public class Fly {

    static double getX( double R, double y ) {
        return Math.abs( R * Math.cos( Math.asin( y/R ) ) );
    }

    static double getY( double R, double x ) {
        return getX( R, x ); // symmetry
    }

    private static double calculateHoleArea( double holeX, double holeY, double squareSide, double innerRadius, int recursionLevel ) {
        double holeX2 = holeX + squareSide;
        double holeY2 = holeY + squareSide;
        final double RADSQR = innerRadius * innerRadius; 
        if ((holeX2 * holeX2 + holeY2 * holeY2) < RADSQR) {
            // easy case
            return squareSide * squareSide;
        } else if (recursionLevel > 3) {
            // approximate, hopefully a small square
            Point2D.Double c1;
            Point2D.Double c2;
            boolean includeUpperLeft = (holeX * holeX + holeY2 * holeY2) < RADSQR;
            boolean includeLowerRight = (holeX2 * holeX2 + holeY * holeY) < RADSQR;
            // find where the radius intersects 
            if (includeUpperLeft) {
                c1 = new Point2D.Double( getX( innerRadius, holeY2 ), holeY2 );
            } else {
                c1 = new Point2D.Double( holeX, getY( innerRadius, holeX ) );
            }
            if (includeLowerRight) {
                c2 = new Point2D.Double( holeX2, getY( innerRadius, holeX2 ));
            } else {
                c2 = new Point2D.Double( getX( innerRadius, holeY ), holeY );
            }
            double areaSum = 0;
            if (includeUpperLeft && includeLowerRight) {
                double width1 = c1.getX() - holeX;
                double height2 = c2.getY() - holeY;
                double box1 = width1 * squareSide;
                double box2 = squareSide * height2;
                double overcount = width1 * height2;
                areaSum += (box1 + box2 - overcount);
            } else if (includeUpperLeft) { // only
                double width = c1.getX() - holeX;
                areaSum += width * squareSide;
            } else if (includeLowerRight) { // only
                double height = c2.getY() - holeY;
                areaSum += squareSide * height;
            }
            double triangleBottom = c2.getX() - c1.getX();
            double triangleHeight = c1.getY() - c2.getY();
            areaSum += (0.5 * triangleBottom * triangleHeight); 
            return areaSum;
        } else {
            // recursion step
            double areaSum = 0;
            int total = 10;
            double miniSquare = squareSide / total;
            double x = holeX;
            for (int i = 0; i < total; ++i) {
                double y = holeY;
                for (int j = 0; j < total; ++j) {
                    if ((x * x) + (y * y) < RADSQR) { 
                        areaSum += calculateHoleArea( x, y, miniSquare, innerRadius, recursionLevel + 1 );
                    }
                    y += miniSquare;
                }
                x += miniSquare;
            }
            return areaSum;
            /*
            Random random = new Random();
            int hits = 0;
            int total = 10000;
            for (int i = 0; i < total; ++i) {
                double x = holeX + (squareSide * random.nextDouble());
                double y = holeY + (squareSide * random.nextDouble());
                double distsqr = x * x +  y * y;
                if (distsqr < RADSQR) ++hits;
            }
            double prob = (double)hits / (double)total;
            return prob * squareSide * squareSide;
            */
        }
    }

    // By symmetry, considering just the upper-right triangle
    private static double calculatePassArea( double innerRadius, double gridInterval, double stringHalfWidth ) {
        double result = 0;
        final int maxHolesPerSide = 1 + (int)(innerRadius / gridInterval);
        for (int nx = 0; nx < maxHolesPerSide; ++nx) {
            for (int ny = 0; ny < maxHolesPerSide; ++ny) {
                double holeX = nx * gridInterval + stringHalfWidth;
                double holeY = ny * gridInterval + stringHalfWidth;
                double distance2origin = Math.sqrt( holeX * holeX  +  holeY * holeY );
                if (distance2origin < innerRadius) {
                    double squareSide = gridInterval - (2 * stringHalfWidth);
                    result += calculateHoleArea( holeX, holeY, squareSide, innerRadius, 0 );
                }
            }
        }
        return result * 4;
    }

    public static void main( String[] args ) throws IOException {
        NumberFormat fmt = new DecimalFormat("0.000000");
        File inputFile = new File( args[0] );
        BufferedReader reader = new BufferedReader( new InputStreamReader( new FileInputStream( inputFile ) ) ); 
        String line = reader.readLine();
        int cases = Integer.parseInt( line.trim() );
        File outputFile = new File( inputFile.getPath() + ".out" ); 
        PrintWriter writer = new PrintWriter( new FileWriter( outputFile ) ); 
        for (int i = 0; i < cases; ++i) {
            StringTokenizer st = new StringTokenizer( reader.readLine() );
            double f = Double.parseDouble( st.nextToken() ); // physical fly radius
            double outerRadius = Double.parseDouble( st.nextToken() );
            double t = Double.parseDouble( st.nextToken() ); // physical ring thickness
            double r = Double.parseDouble( st.nextToken() ); // half-width of strings 
            double g = Double.parseDouble( st.nextToken() ); // max physical hole dimension
            // Calculating the area the fly's center can pass through
            double gridInterval = g + 2 * r;
            double squareSide = g - 2 * f;
            double innerRadius = outerRadius - t - f;
            double effectiveStringWidth = r + f;
            double passArea = calculatePassArea( innerRadius, gridInterval, effectiveStringWidth );
            double fullArea = Math.PI * outerRadius * outerRadius; 
            double passProb = passArea / fullArea;
            passProb = Math.min( passProb, 1 );
            passProb = Math.max( passProb, 0 );
            double killProb = 1 - passProb;
            String resultLine = "Case #" + (i + 1) + ": " + fmt.format( killProb );
            System.out.println( resultLine );
            writer.println( resultLine ); 
        }
        reader.close();
        writer.close();
    }
}
