import java.io.*;
import java.io.ObjectInputStream.GetField;
import java.util.Scanner;

public class BotTrust {
	public static enum BotType {
		Orange, Blue
	}

	public static class Instruction {
		public final BotType botType;
		public final int button;
		private final int index;

		public Instruction(BotType botType, int button, int index) {
			this.botType = botType;
			this.button = button;
			this.index = index;
		}

		public Instruction(String botType, int button, int index) {
			this.botType = botType.equalsIgnoreCase("o") ? BotType.Orange
					: BotType.Blue;
			this.button = button;
			this.index = index;
		}
		
		public boolean isMoreUrgentThan(Instruction other) {
			return other == null || index < other.index;
		}
		public boolean isLessUrgentThan(Instruction other) {
			return other == null || index > other.index;
		}

		public String toString() {
			return botType + " to push button " + button;
		}

	}

	public static class Bot {
		public static enum State {
			WAITING, FORWARD, BACKWARD, IDLE, DONE
		}

		BotType type;
		State state = State.IDLE;

		int position;
		int target;

		public Bot(BotType type) {
			this.type = type;
		}

		public void update() {
			if (state == State.FORWARD)
				position++;
			else if (state == State.BACKWARD)
				position--;
			
			if (inPosition()) {
				state = State.WAITING;
			}
		}

		public void pushButton() {
			state = State.IDLE;
			//System.out.println(type + " pushed button " + target);
		}

		private void setTarget(int target) {
			if (state != State.IDLE) {
				throw new IllegalStateException("busy!");
			} else if (target == position) {
				state = State.WAITING;
			} else {
				this.target = target;

				if (target > position)
					state = State.FORWARD;
				else if (target < position)
					state = State.BACKWARD;
			}
		}
		
		public void instruct(Instruction i) {
			//System.out.println(type + " recieved: "+i);
			if(i == null)
				state = State.DONE;
			else if(i.botType == type)
				setTarget(i.button);
		}

		public boolean inPosition() {
			return (state == State.FORWARD || state == State.BACKWARD) && target == position;
		}

		public State getState() {
			return state;
		}

		public boolean isIdle() {
			return state == State.IDLE;
		}

		public boolean isWaiting() {
			return state == State.WAITING;
		}
		public boolean isDone() {
			return state == State.DONE;
		}


		public String toString() {
			return type + " bot is at button " + position
					+ " and going to button " + target;
		}
	}

	public static Instruction getNextInstruction(BotType color,
			Instruction[] instructions, Instruction last) {
		for (Instruction i : instructions) {
			if (i.isLessUrgentThan(last) && i.botType == color) {
				return i;
			}
		}
		return null;
	}

	public static int solveTestCase(Instruction[] instructions)
			throws InterruptedException {
		Bot atlas = new Bot(BotType.Blue);
		Bot pbody = new Bot(BotType.Orange);
		int time = 0;
		Instruction blueInstruction = null;
		Instruction orangeInstruction = null;
		while (true) {
			if (pbody.isIdle()) {
				orangeInstruction = getNextInstruction(BotType.Orange, instructions, orangeInstruction);
				pbody.instruct(orangeInstruction);
			}
			
			if (atlas.isIdle()) {
				blueInstruction = getNextInstruction(BotType.Blue, instructions, blueInstruction);
				atlas.instruct(blueInstruction);
			}

			if (pbody.isWaiting() && orangeInstruction.isMoreUrgentThan(blueInstruction))
				pbody.pushButton();
			
			if (atlas.isWaiting() && blueInstruction.isMoreUrgentThan(orangeInstruction))
				atlas.pushButton();

			if(atlas.isDone() && pbody.isDone()) return time-1;
			pbody.update();
			atlas.update();
			time++;
		}

		// return instructions.length;
	}

	public static void main(String[] args) throws IOException,
			InterruptedException {
		final String fileName = "small";
		
		FileInputStream inFile = new FileInputStream(fileName+".in");
		Scanner in = new Scanner(inFile);
		
		FileOutputStream outFile = new FileOutputStream(fileName+".out");
		PrintWriter out = new PrintWriter(outFile);

		int testCases = in.nextInt();

		for (int i = 1; i <= testCases; i++) {
			Instruction[] instructions = new Instruction[in.nextInt()];
			for (int j = 0; j < instructions.length; j++) {
				instructions[j] = new Instruction(in.next(), in.nextInt(), j);
			}
			out.println("Case #" + i + ": "
					+ solveTestCase(instructions));
		}
		out.flush();
		out.close();
	}
}
